@@ -9,7 +9,7 @@
  *             Tomas Frydrych <tf@openedhand.com>
  *
  * Copyright (C) 2006, 2007, 2008 OpenedHand
- * Copyright (C) 2009 Intel Corp.
+ * Copyright (C) 2009, 2010 Intel Corp.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,15 +22,12 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
  * SECTION:clutter-alpha
- * @short_description: A class for calculating an alpha value as a function
- * of time.
+ * @short_description: A class for calculating a value as a function of time
  *
  * #ClutterAlpha is a class for calculating an floating point value
  * dependent only on the position of a #ClutterTimeline.
@@ -64,6 +61,39 @@
  *   <graphic fileref="easing-modes.png" format="PNG"/>
  * </figure>
  *
+ * <refsect2 id="ClutterAlpha-script">
+ *   <title>ClutterAlpha custom properties for #ClutterScript</title>
+ *   <para>#ClutterAlpha defines a custom "function" property for
+ *   #ClutterScript which allows to reference a custom alpha function
+ *   available in the source code. Setting the "function" property
+ *   is equivalent to calling clutter_alpha_set_func() with the
+ *   specified function name. No user data or #GDestroyNotify is
+ *   available to be passed.</para>
+ *   <example id="ClutterAlpha-script-example">
+ *     <title>Defining a ClutterAlpha in ClutterScript</title>
+ *     <para>The following JSON fragment defines a #ClutterAlpha
+ *     using a #ClutterTimeline with id "sine-timeline" and an alpha
+ *     function called my_sine_alpha(). The defined #ClutterAlpha
+ *     instance can be reused in multiple #ClutterBehaviour
+ *     definitions or for #ClutterAnimation definitions.</para>
+ *     <programlisting><![CDATA[
+ * {
+ *   "id" : "sine-alpha",
+ *   "timeline" : {
+ *     "id" : "sine-timeline",
+ *     "duration" : 500,
+ *     "loop" : true
+ *   },
+ *   "function" : "my_sine_alpha"
+ * }
+ *    ]]></programlisting>
+ *   </example>
+ *   <para>For the way to define the #ClutterAlpha:mode property
+ *   inside a ClutterScript fragment, see <link
+ *   linkend="clutter-AnimationMode-Script">the corresponding section</link>
+ *   in #ClutterAnimation.</para>
+ * </refsect2>
+ *
  * Since: 0.2
  */
 
@@ -73,15 +103,24 @@
 
 #include <math.h>
 
+#include <gmodule.h>
+
 #include "clutter-alpha.h"
 #include "clutter-debug.h"
 #include "clutter-enum-types.h"
 #include "clutter-main.h"
 #include "clutter-marshal.h"
 #include "clutter-private.h"
+#include "clutter-scriptable.h"
+#include "clutter-script-private.h"
 
-G_DEFINE_TYPE (ClutterAlpha, clutter_alpha, G_TYPE_INITIALLY_UNOWNED);
+static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
 
+G_DEFINE_TYPE_WITH_CODE (ClutterAlpha,
+                         clutter_alpha,
+                         G_TYPE_INITIALLY_UNOWNED,
+                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                                clutter_scriptable_iface_init));
 
 struct _ClutterAlphaPrivate
 {
@@ -122,11 +161,7 @@ clutter_alpha_set_property (GObject      *object,
 			    const GValue *value, 
 			    GParamSpec   *pspec)
 {
-  ClutterAlpha *alpha;
-  ClutterAlphaPrivate *priv;
-
-  alpha = CLUTTER_ALPHA (object);
-  priv = alpha->priv;
+  ClutterAlpha *alpha = CLUTTER_ALPHA (object);
 
   switch (prop_id) 
     {
@@ -150,11 +185,7 @@ clutter_alpha_get_property (GObject    *object,
 			    GValue     *value, 
 			    GParamSpec *pspec)
 {
-  ClutterAlpha        *alpha;
-  ClutterAlphaPrivate *priv;
-
-  alpha = CLUTTER_ALPHA (object);
-  priv = alpha->priv;
+  ClutterAlphaPrivate *priv = CLUTTER_ALPHA (object)->priv;
 
   switch (prop_id) 
     {
@@ -197,6 +228,88 @@ clutter_alpha_dispose (GObject *object)
   G_OBJECT_CLASS (clutter_alpha_parent_class)->dispose (object);
 }
 
+static ClutterAlphaFunc
+resolve_alpha_func (const gchar *name)
+{
+  static GModule *module = NULL;
+  ClutterAlphaFunc func;
+
+  CLUTTER_NOTE (SCRIPT, "Looking up '%s' alpha function", name);
+
+  if (G_UNLIKELY (module == NULL))
+    module = g_module_open (NULL, 0);
+
+  if (g_module_symbol (module, name, (gpointer) &func))
+    {
+      CLUTTER_NOTE (SCRIPT, "Found '%s' alpha function in the symbols table",
+                    name);
+      return func;
+    }
+
+  return NULL;
+}
+
+static void
+clutter_alpha_set_custom_property (ClutterScriptable *scriptable,
+                                   ClutterScript     *script,
+                                   const gchar       *name,
+                                   const GValue      *value)
+{
+  if (strncmp (name, "function", 8) == 0)
+    {
+      g_assert (G_VALUE_HOLDS (value, G_TYPE_POINTER));
+      if (g_value_get_pointer (value) != NULL)
+        {
+          clutter_alpha_set_func (CLUTTER_ALPHA (scriptable),
+                                  g_value_get_pointer (value),
+                                  NULL, NULL);
+        }
+    }
+  else
+    g_object_set_property (G_OBJECT (scriptable), name, value);
+}
+
+static gboolean
+clutter_alpha_parse_custom_node (ClutterScriptable *scriptable,
+                                 ClutterScript     *script,
+                                 GValue            *value,
+                                 const gchar       *name,
+                                 JsonNode          *node)
+{
+  if (strncmp (name, "function", 8) == 0)
+    {
+      const gchar *func_name = json_node_get_string (node);
+
+      g_value_init (value, G_TYPE_POINTER);
+      g_value_set_pointer (value, resolve_alpha_func (func_name));
+
+      return TRUE;
+    }
+
+  /* we need to do this because we use gulong in place
+   * of ClutterAnimationMode for ClutterAlpha:mode
+   */
+  if (strncmp (name, "mode", 4) == 0)
+    {
+      gulong mode;
+
+      mode = clutter_script_resolve_animation_mode (node);
+
+      g_value_init (value, G_TYPE_ULONG);
+      g_value_set_ulong (value, mode);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static void
+clutter_scriptable_iface_init (ClutterScriptableIface *iface)
+{
+  iface->parse_custom_node = clutter_alpha_parse_custom_node;
+  iface->set_custom_property = clutter_alpha_set_custom_property;
+}
 
 static void
 clutter_alpha_class_init (ClutterAlphaClass *klass)
@@ -345,7 +458,7 @@ clutter_alpha_set_closure_internal (ClutterAlpha *alpha,
   /* set the marshaller */
   if (G_CLOSURE_NEEDS_MARSHAL (closure))
     {
-      GClosureMarshal marshal = clutter_marshal_DOUBLE__VOID;
+      GClosureMarshal marshal = _clutter_marshal_DOUBLE__VOID;
 
       g_closure_set_marshal (closure, marshal);
     }
