@@ -3,9 +3,12 @@
  *
  * An OpenGL based 'interactive canvas' library.
  *
- * Authored By Matthew Allum  <mallum@openedhand.com>
+ * Authored By:
+ *      Matthew Allum <mallum@openedhand.com>
+ *      Emmanuele Bassi <ebassi@linux.intel.com>
  *
- * Copyright (C) 2006 OpenedHand
+ * Copyright (C) 2006, 2007, 2008 OpenedHand Ltd
+ * Copyright (C) 2009, 2010 Intel Corp
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,9 +21,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -59,24 +60,19 @@ G_DEFINE_ABSTRACT_TYPE (ClutterBackend, clutter_backend, G_TYPE_OBJECT);
 
 struct _ClutterBackendPrivate
 {
-  /* settings */
-  guint double_click_time;
-  guint double_click_distance;
-
-  gdouble resolution;
-
-  gfloat units_per_em;
-  gint32 units_serial;
-
   cairo_font_options_t *font_options;
 
   gchar *font_name;
+
+  gfloat units_per_em;
+  gint32 units_serial;
 };
 
 enum
 {
   RESOLUTION_CHANGED,
   FONT_CHANGED,
+  SETTINGS_CHANGED,
 
   LAST_SIGNAL
 };
@@ -86,7 +82,6 @@ static guint backend_signals[LAST_SIGNAL] = { 0, };
 static void
 clutter_backend_dispose (GObject *gobject)
 {
-  ClutterBackendPrivate *priv = CLUTTER_BACKEND (gobject)->priv;
   ClutterMainContext *clutter_context;
 
   clutter_context = _clutter_context_get_default ();
@@ -100,13 +95,20 @@ clutter_backend_dispose (GObject *gobject)
       clutter_context->events_queue = NULL;
     }
 
-  g_free (priv->font_name);
-
-  clutter_backend_set_font_options (CLUTTER_BACKEND (gobject), NULL);
-
   G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
 }
 
+static void
+clutter_backend_finalize (GObject *gobject)
+{
+  ClutterBackend *backend = CLUTTER_BACKEND (gobject);
+
+  g_free (backend->priv->font_name);
+  clutter_backend_set_font_options (backend, NULL);
+
+  G_OBJECT_CLASS (clutter_backend_parent_class)->finalize (gobject);
+}
+
 static gfloat
 get_units_per_em (ClutterBackend       *backend,
                   PangoFontDescription *font_desc)
@@ -165,11 +167,22 @@ static void
 clutter_backend_real_resolution_changed (ClutterBackend *backend)
 {
   ClutterBackendPrivate *priv = backend->priv;
+  ClutterMainContext *context;
+  ClutterSettings *settings;
+  gint dpi;
+
+  context = _clutter_context_get_default ();
+
+  settings = clutter_settings_get_default ();
+  g_object_get (settings, "font-dpi", &dpi, NULL);
+
+  if (context->font_map != NULL)
+    cogl_pango_font_map_set_resolution (context->font_map, dpi / 1024.0);
 
   priv->units_per_em = get_units_per_em (backend, NULL);
   priv->units_serial += 1;
 
-  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", backend->priv->units_per_em);
+  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", priv->units_per_em);
 }
 
 static void
@@ -180,7 +193,7 @@ clutter_backend_real_font_changed (ClutterBackend *backend)
   priv->units_per_em = get_units_per_em (backend, NULL);
   priv->units_serial += 1;
 
-  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", backend->priv->units_per_em);
+  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", priv->units_per_em);
 }
 
 static void
@@ -189,6 +202,7 @@ clutter_backend_class_init (ClutterBackendClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->dispose = clutter_backend_dispose;
+  gobject_class->finalize = clutter_backend_finalize;
 
   g_type_class_add_private (gobject_class, sizeof (ClutterBackendPrivate));
 
@@ -198,7 +212,7 @@ clutter_backend_class_init (ClutterBackendClass *klass)
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (ClutterBackendClass, resolution_changed),
                   NULL, NULL,
-                  clutter_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   backend_signals[FONT_CHANGED] =
@@ -207,7 +221,16 @@ clutter_backend_class_init (ClutterBackendClass *klass)
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (ClutterBackendClass, font_changed),
                   NULL, NULL,
-                  clutter_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
+  backend_signals[SETTINGS_CHANGED] =
+    g_signal_new (I_("settings-changed"),
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (ClutterBackendClass, settings_changed),
+                  NULL, NULL,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   klass->resolution_changed = clutter_backend_real_resolution_changed;
@@ -221,8 +244,6 @@ clutter_backend_init (ClutterBackend *backend)
 
   priv = backend->priv = CLUTTER_BACKEND_GET_PRIVATE (backend);
 
-  priv->resolution = -1.0;
-
   priv->units_per_em = -1.0;
   priv->units_serial = 1;
 }
@@ -279,16 +300,18 @@ _clutter_backend_create_stage (ClutterBackend  *backend,
   ClutterStageManager *stage_manager;
   ClutterStageWindow *stage_window;
 
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), FALSE);
-  g_return_val_if_fail (CLUTTER_IS_STAGE (wrapper), FALSE);
+  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), NULL);
+  g_return_val_if_fail (CLUTTER_IS_STAGE (wrapper), NULL);
 
   stage_manager = clutter_stage_manager_get_default ();
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
-  if (klass->create_stage)
+  if (klass->create_stage != NULL)
     stage_window = klass->create_stage (backend, wrapper, error);
+  else
+    stage_window = NULL;
 
-  if (!stage_window)
+  if (stage_window == NULL)
     return NULL;
 
   g_assert (CLUTTER_IS_STAGE_WINDOW (stage_window));
@@ -308,7 +331,7 @@ _clutter_backend_redraw (ClutterBackend *backend,
                           "Increments for each _clutter_backend_redraw call",
                           0 /* no application private data */);
   CLUTTER_STATIC_TIMER (redraw_timer,
-                        "Mainloop", /* parent */
+                        "Master Clock", /* parent */
                         "Redrawing",
                         "The time spent redrawing everything",
                         0 /* no application private data */);
@@ -421,6 +444,7 @@ ClutterFeatureFlags
 _clutter_backend_get_features (ClutterBackend *backend)
 {
   ClutterBackendClass *klass;
+  GError *error;
 
   g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), 0);
 
@@ -430,8 +454,25 @@ _clutter_backend_get_features (ClutterBackend *backend)
    * GL context first and the ask for features. if the
    * context already exists this should be a no-op
    */
+  error = NULL;
   if (klass->create_context)
-    klass->create_context (backend, NULL);
+    {
+      gboolean res;
+
+      res = klass->create_context (backend, &error);
+      if (!res)
+        {
+          if (error)
+            {
+              g_critical ("Unable to create a context: %s", error->message);
+              g_error_free (error);
+            }
+          else
+            g_critical ("Unable to create a context: unknown error");
+
+          return 0;
+        }
+    }
 
   if (klass->get_features)
     return klass->get_features (backend);
@@ -510,14 +551,16 @@ clutter_get_default_backend (void)
  * verify whether it's a double click event or not.
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:double-click-time instead
  */
 void
 clutter_backend_set_double_click_time (ClutterBackend *backend,
                                        guint           msec)
 {
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
+  ClutterSettings *settings = clutter_settings_get_default ();
 
-  backend->priv->double_click_time = msec;
+  g_object_set (settings, "double-click-time", msec, NULL);
 }
 
 /**
@@ -530,13 +573,18 @@ clutter_backend_set_double_click_time (ClutterBackend *backend,
  * Return value: a time in milliseconds
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:double-click-time instead
  */
 guint
 clutter_backend_get_double_click_time (ClutterBackend *backend)
 {
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), 0);
+  ClutterSettings *settings = clutter_settings_get_default ();
+  gint retval;
 
-  return backend->priv->double_click_time;
+  g_object_get (settings, "double-click-time", &retval, NULL);
+
+  return retval;
 }
 
 /**
@@ -547,14 +595,16 @@ clutter_backend_get_double_click_time (ClutterBackend *backend)
  * Sets the maximum distance used to verify a double click event.
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:double-click-distance instead
  */
 void
 clutter_backend_set_double_click_distance (ClutterBackend *backend,
                                            guint           distance)
 {
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
-  
-  backend->priv->double_click_distance = distance;
+  ClutterSettings *settings = clutter_settings_get_default ();
+
+  g_object_set (settings, "double-click-distance", distance, NULL);
 }
 
 /**
@@ -566,13 +616,18 @@ clutter_backend_set_double_click_distance (ClutterBackend *backend,
  * Return value: a distance, in pixels.
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:double-click-distance instead
  */
 guint
 clutter_backend_get_double_click_distance (ClutterBackend *backend)
 {
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), 0);
+  ClutterSettings *settings = clutter_settings_get_default ();
+  gint retval;
 
-  return backend->priv->double_click_distance;
+  g_object_get (settings, "double-click-distance", &retval, NULL);
+
+  return retval;
 }
 
 /**
@@ -589,35 +644,42 @@ clutter_backend_get_double_click_distance (ClutterBackend *backend)
  * Applications should never need to call this function.
  *
  * Since: 0.4
+ *
+ * Deprecated: Use #ClutterSettings:font-dpi instead
  */
 void
 clutter_backend_set_resolution (ClutterBackend *backend,
                                 gdouble         dpi)
 {
-  ClutterBackendPrivate *priv;
+  ClutterSettings *settings;
+  gint resolution;
 
   g_return_if_fail (CLUTTER_IS_BACKEND (backend));
 
-  priv = backend->priv;
-
   if (dpi < 0)
-    dpi = -1.0;
+    resolution = -1;
+  else
+    resolution = dpi * 1024;
 
-  priv->resolution = dpi;
-
-  if (CLUTTER_CONTEXT ()->font_map)
-    cogl_pango_font_map_set_resolution (CLUTTER_CONTEXT ()->font_map, dpi);
-
-  g_signal_emit (backend, backend_signals[RESOLUTION_CHANGED], 0);
+  settings = clutter_settings_get_default ();
+  g_object_set (settings, "font-dpi", resolution, NULL);
 }
 
 /**
  * clutter_backend_get_resolution:
  * @backend: a #ClutterBackend
  *
- * Gets the resolution for font handling on the screen; see
- * clutter_backend_set_resolution() for full details.
- * 
+ * Gets the resolution for font handling on the screen.
+ *
+ * The resolution is a scale factor between points specified in a
+ * #PangoFontDescription and cairo units. The default value is 96.0,
+ * meaning that a 10 point font will be 13 units
+ * high (10 * 96. / 72. = 13.3).
+ *
+ * Clutter will set the resolution using the current backend when
+ * initializing; the resolution is also stored in the
+ * #ClutterSettings:font-dpi property.
+ *
  * Return value: the current resolution, or -1 if no resolution
  *   has been set.
  *
@@ -626,9 +688,15 @@ clutter_backend_set_resolution (ClutterBackend *backend,
 gdouble
 clutter_backend_get_resolution (ClutterBackend *backend)
 {
+  ClutterSettings *settings;
+  gint resolution;
+
   g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), -1.0);
 
-  return backend->priv->resolution;
+  settings = clutter_settings_get_default ();
+  g_object_get (settings, "font-dpi", &resolution, NULL);
+
+  return resolution / 1024.0;
 }
 
 /**
@@ -649,7 +717,7 @@ clutter_backend_get_resolution (ClutterBackend *backend)
  * Since: 0.8
  */
 void
-clutter_backend_set_font_options (ClutterBackend       *backend,
+clutter_backend_set_font_options (ClutterBackend             *backend,
                                   const cairo_font_options_t *options)
 {
   ClutterBackendPrivate *priv;
@@ -721,25 +789,16 @@ clutter_backend_get_font_options (ClutterBackend *backend)
  * be parsed by the pango_font_description_from_string() function.
  *
  * Since: 1.0
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:font-name instead
  */
 void
 clutter_backend_set_font_name (ClutterBackend *backend,
                                const gchar    *font_name)
 {
-  ClutterBackendPrivate *priv;
+  ClutterSettings *settings = clutter_settings_get_default ();
 
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
-
-  priv = backend->priv;
-
-  g_free (priv->font_name);
-
-  if (font_name == NULL || *font_name == '\0')
-    priv->font_name = g_strdup (DEFAULT_FONT_NAME);
-  else
-    priv->font_name = g_strdup (font_name);
-
-  g_signal_emit (backend, backend_signals[FONT_CHANGED], 0);
+  g_object_set (settings, "font-name", font_name, NULL);
 }
 
 /**
@@ -753,25 +812,26 @@ clutter_backend_set_font_name (ClutterBackend *backend,
  *   owned by the #ClutterBackend and should never be modified or freed
  *
  * Since: 1.0
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:font-name instead
  */
 G_CONST_RETURN gchar *
 clutter_backend_get_font_name (ClutterBackend *backend)
 {
   ClutterBackendPrivate *priv;
+  ClutterSettings *settings;
 
   g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), NULL);
 
   priv = backend->priv;
 
-  if (G_LIKELY (priv->font_name))
-    return priv->font_name;
+  settings = clutter_settings_get_default ();
 
-  /* if we have never been called then we need to set the
-   * default font and update everything that relies on the
-   * ::font-changed signal
+  /* XXX yuck. but we return a const pointer, so we need to
+   * store it in the backend
    */
-  priv->font_name = g_strdup (DEFAULT_FONT_NAME);
-  g_signal_emit (backend, backend_signals[FONT_CHANGED], 0);
+  g_free (priv->font_name);
+  g_object_get (settings, "font-name", &priv->font_name, NULL);
 
   return priv->font_name;
 }
