@@ -503,7 +503,7 @@ clutter_binding_pool_install_action (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
       g_closure_set_marshal (closure, marshal);
     }
 
@@ -569,7 +569,7 @@ clutter_binding_pool_install_closure (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
       g_closure_set_marshal (closure, marshal);
     }
 
@@ -637,7 +637,7 @@ clutter_binding_pool_override_action (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
       g_closure_set_marshal (closure, marshal);
     }
 }
@@ -697,7 +697,7 @@ clutter_binding_pool_override_closure (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
       g_closure_set_marshal (closure, marshal);
     }
 }
