@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #if !defined(__COGL_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
@@ -32,6 +32,8 @@
 
 G_BEGIN_DECLS
 
+typedef struct _CoglBitmap CoglBitmap;
+
 /**
  * SECTION:cogl-bitmap
  * @short_description: Fuctions for loading images
@@ -51,13 +53,14 @@ G_BEGIN_DECLS
  * Loads an image file from disk. This function can be safely called from
  * within a thread.
  *
- * Return value: a #CoglHandle to the new loaded image data, or
- *   %COGL_INVALID_HANDLE if loading the image failed.
+ * Return value: a #CoglBitmap to the new loaded image data, or
+ *   %NULL if loading the image failed.
  *
  * Since: 1.0
  */
-CoglHandle      cogl_bitmap_new_from_file      (const gchar  *filename,
-                                                GError      **error);
+CoglBitmap *
+cogl_bitmap_new_from_file (const char *filename,
+                           GError **error);
 
 /**
  * cogl_bitmap_get_size_from_file:
@@ -72,9 +75,10 @@ CoglHandle      cogl_bitmap_new_from_file      (const gchar  *filename,
  *
  * Since: 1.0
  */
-gboolean        cogl_bitmap_get_size_from_file (const gchar  *filename,
-                                                gint         *width,
-                                                gint         *height);
+gboolean
+cogl_bitmap_get_size_from_file (const char *filename,
+                                int *width,
+                                int *height);
 
 /**
  * cogl_is_bitmap:
@@ -87,7 +91,8 @@ gboolean        cogl_bitmap_get_size_from_file (const gchar  *filename,
  *
  * Since: 1.0
  */
-gboolean        cogl_is_bitmap                 (CoglHandle    handle);
+gboolean
+cogl_is_bitmap (CoglHandle handle);
 
 G_END_DECLS
 
