@@ -33,11 +33,25 @@
 #include "cogl-texture-private.h"
 #include "cogl-material-private.h"
 #include "cogl-framebuffer-private.h"
+#include "cogl-path-private.h"
 
 #include <string.h>
 
+#ifdef HAVE_COGL_GL
+#define glActiveTexture _context->drv.pf_glActiveTexture
+#endif
+
+/* This isn't defined in the GLES headers */
+#ifndef GL_POINT_SPRITE
+#define GL_POINT_SPRITE 0x8861
+#endif
+
 extern void
 _cogl_create_context_driver (CoglContext *context);
+extern void
+_cogl_create_context_winsys (CoglContext *context);
+extern void
+_cogl_destroy_context_winsys (CoglContext *context);
 
 static CoglContext *_context = NULL;
 static gboolean gl_is_indirect = FALSE;
@@ -48,6 +62,7 @@ cogl_create_context (void)
   GLubyte default_texture_data[] = { 0xff, 0xff, 0xff, 0x0 };
   unsigned long  enable_flags = 0;
   CoglHandle window_buffer;
+  int i;
 
   if (_context != NULL)
     return FALSE;
@@ -57,17 +72,23 @@ cogl_create_context (void)
 
   /* Init default values */
   _context->feature_flags = 0;
+  _context->feature_flags_private = 0;
   _context->features_cached = FALSE;
 
+  _context->texture_types = NULL;
+  _context->buffer_types = NULL;
+
   /* Initialise the driver specific state */
   /* TODO: combine these two into one function */
   _cogl_create_context_driver (_context);
   _cogl_features_init ();
 
+  _cogl_create_context_winsys (_context);
+
   _cogl_material_init_default_material ();
+  _cogl_material_init_default_layers ();
 
   _context->enable_flags = 0;
-  _context->color_alpha = 0;
 
   _context->enable_backface_culling = FALSE;
   _context->flushed_front_winding = COGL_FRONT_WINDING_COUNTER_CLOCKWISE;
@@ -79,10 +100,22 @@ cogl_create_context (void)
   cogl_matrix_scale (&_context->y_flip_matrix, 1, -1, 1);
 
   _context->flushed_matrix_mode = COGL_MATRIX_MODELVIEW;
-  _context->texture_units = NULL;
+
+  _context->texture_units =
+    g_array_new (FALSE, FALSE, sizeof (CoglTextureUnit));
+
+  /* See cogl-material.c for more details about why we leave texture unit 1
+   * active by default... */
+  _context->active_texture_unit = 1;
+  GE (glActiveTexture (GL_TEXTURE1));
+
+  _context->legacy_fog_state.enabled = FALSE;
 
   _context->simple_material = cogl_material_new ();
   _context->source_material = NULL;
+  _context->arbfp_source_buffer = g_string_new ("");
+
+  _context->legacy_state_set = 0;
 
   _context->default_gl_texture_2d_tex = COGL_INVALID_HANDLE;
   _context->default_gl_texture_rect_tex = COGL_INVALID_HANDLE;
@@ -91,14 +124,44 @@ cogl_create_context (void)
   _context->logged_vertices = g_array_new (FALSE, FALSE, sizeof (GLfloat));
 
   _context->current_material = NULL;
-  _context->current_material_flags = 0;
-  memset (&_context->current_material_flush_options,
-          0, sizeof (CoglMaterialFlushOptions));
-  _context->current_layers = g_array_new (FALSE, FALSE,
-                                          sizeof (CoglLayerInfo));
-  _context->n_texcoord_arrays_enabled = 0;
+  _context->current_material_changes_since_flush = 0;
+  _context->current_material_skip_gl_color = FALSE;
+
+  _context->material0_nodes =
+    g_array_sized_new (FALSE, FALSE, sizeof (CoglHandle), 20);
+  _context->material1_nodes =
+    g_array_sized_new (FALSE, FALSE, sizeof (CoglHandle), 20);
+
+  _cogl_bitmask_init (&_context->texcoord_arrays_enabled);
+  _cogl_bitmask_init (&_context->temp_bitmask);
+  _cogl_bitmask_init (&_context->texcoord_arrays_to_disable);
+
+  _context->max_texture_units = -1;
+  _context->max_texture_image_units = -1;
+  _context->max_activateable_texture_units = -1;
+
+  _context->current_program = COGL_INVALID_HANDLE;
+
+  _context->current_use_program_type = COGL_MATERIAL_PROGRAM_TYPE_FIXED;
+  _context->current_gl_program = 0;
+
+  _context->gl_blend_enable_cache = FALSE;
+
+  _context->depth_test_enabled_cache = FALSE;
+  _context->depth_test_function_cache = COGL_DEPTH_TEST_FUNCTION_LESS;
+  _context->depth_writing_enabled_cache = TRUE;
+  _context->depth_range_near_cache = 0;
+  _context->depth_range_far_cache = 1;
+
+  _context->point_size_cache = 1.0f;
+
+  _context->legacy_depth_test_enabled = FALSE;
+
+  for (i = 0; i < COGL_BUFFER_BIND_TARGET_COUNT; i++)
+    _context->current_buffer[i] = NULL;
 
   _context->framebuffer_stack = _cogl_create_framebuffer_stack ();
+
   window_buffer = _cogl_onscreen_new ();
   cogl_set_framebuffer (window_buffer);
   /* XXX: the deprecated _cogl_set_draw_buffer API expects to
@@ -108,8 +171,7 @@ cogl_create_context (void)
   _context->dirty_bound_framebuffer = TRUE;
   _context->dirty_gl_viewport = TRUE;
 
-  _context->path_nodes = g_array_new (FALSE, FALSE, sizeof (CoglPathNode));
-  _context->last_path = 0;
+  _context->current_path = _cogl_path_new ();
   _context->stencil_material = cogl_material_new ();
 
   _context->in_begin_gl_block = FALSE;
@@ -120,6 +182,12 @@ cogl_create_context (void)
 
   _context->texture_download_material = COGL_INVALID_HANDLE;
 
+  /* The default for GL_ALPHA_TEST is to always pass which is equivalent to
+   * the test being disabled therefore we assume that for all drivers there
+   * will be no performance impact if we always leave the test enabled which
+   * makes things a bit simpler for us. */
+  GE (glEnable (GL_ALPHA_TEST));
+
   /* Create default textures used for fall backs */
   _context->default_gl_texture_2d_tex =
     cogl_texture_new_from_data (1, /* width */
@@ -141,35 +209,40 @@ cogl_create_context (void)
                                 default_texture_data);
 
   cogl_set_source (_context->simple_material);
-  _cogl_material_flush_gl_state (_context->source_material, NULL);
-  enable_flags =
-    _cogl_material_get_cogl_enable_flags (_context->source_material);
-  cogl_enable (enable_flags);
+  _cogl_material_flush_gl_state (_context->source_material, FALSE);
+  _cogl_enable (enable_flags);
   _cogl_flush_face_winding ();
 
   _context->atlas = NULL;
   _context->atlas_texture = COGL_INVALID_HANDLE;
 
-  _context->current_pbo = NULL;
-
-  _context->max_texture_units = -1;
+  /* As far as I can tell, GL_POINT_SPRITE doesn't have any effect
+     unless GL_COORD_REPLACE is enabled for an individual
+     layer. Therefore it seems like it should be ok to just leave it
+     enabled all the time instead of having to have a set property on
+     each material to track whether any layers have point sprite
+     coords enabled */
+  if (cogl_features_available (COGL_FEATURE_POINT_SPRITE))
+    GE (glEnable (GL_POINT_SPRITE));
 
   return TRUE;
 }
 
 void
-_cogl_destroy_context ()
+_cogl_destroy_context (void)
 {
 
   if (_context == NULL)
     return;
 
+  _cogl_destroy_context_winsys (_context);
+
   _cogl_destroy_texture_units ();
 
   _cogl_free_framebuffer_stack (_context->framebuffer_stack);
 
-  if (_context->path_nodes)
-    g_array_free (_context->path_nodes, TRUE);
+  if (_context->current_path)
+    cogl_handle_unref (_context->current_path);
 
   if (_context->default_gl_texture_2d_tex)
     cogl_handle_unref (_context->default_gl_texture_2d_tex);
@@ -184,9 +257,6 @@ _cogl_destroy_context ()
   if (_context->logged_vertices)
     g_array_free (_context->logged_vertices, TRUE);
 
-  if (_context->current_layers)
-    g_array_free (_context->current_layers, TRUE);
-
   if (_context->quad_indices_byte)
     cogl_handle_unref (_context->quad_indices_byte);
   if (_context->quad_indices_short)
@@ -195,16 +265,30 @@ _cogl_destroy_context ()
   if (_context->default_material)
     cogl_handle_unref (_context->default_material);
 
+  if (_context->dummy_layer_dependant)
+    cogl_handle_unref (_context->dummy_layer_dependant);
+  if (_context->default_layer_n)
+    cogl_handle_unref (_context->default_layer_n);
+  if (_context->default_layer_0)
+    cogl_handle_unref (_context->default_layer_0);
+
   if (_context->atlas)
     _cogl_atlas_free (_context->atlas);
   if (_context->atlas_texture)
     cogl_handle_unref (_context->atlas_texture);
 
+  _cogl_bitmask_destroy (&_context->texcoord_arrays_enabled);
+  _cogl_bitmask_destroy (&_context->temp_bitmask);
+  _cogl_bitmask_destroy (&_context->texcoord_arrays_to_disable);
+
+  g_slist_free (_context->texture_types);
+  g_slist_free (_context->buffer_types);
+
   g_free (_context);
 }
 
 CoglContext *
-_cogl_context_get_default ()
+_cogl_context_get_default (void)
 {
   /* Create if doesn't exist yet */
   if (_context == NULL)
