@@ -43,6 +43,11 @@ G_BEGIN_DECLS
  * blended together.
  */
 
+typedef struct _CoglMaterial	      CoglMaterial;
+typedef struct _CoglMaterialLayer     CoglMaterialLayer;
+
+#define COGL_MATERIAL(OBJECT) ((CoglMaterial *)OBJECT)
+
 /**
  * CoglMaterialFilter:
  * @COGL_MATERIAL_FILTER_NEAREST: Measuring in manhatten distance from the,
@@ -79,21 +84,61 @@ typedef enum {
   COGL_MATERIAL_FILTER_LINEAR_MIPMAP_LINEAR = GL_LINEAR_MIPMAP_LINEAR
 } CoglMaterialFilter;
 
+/**
+ * CoglMaterialWrapMode:
+ * @COGL_MATERIAL_WRAP_MODE_REPEAT: The texture will be repeated. This
+ *   is useful for example to draw a tiled background.
+ * @COGL_MATERIAL_WRAP_MODE_CLAMP_TO_EDGE: The coordinates outside the
+ *   range 0→1 will sample copies of the edge pixels of the
+ *   texture. This is useful to avoid artifacts if only one copy of
+ *   the texture is being rendered.
+ * @COGL_MATERIAL_WRAP_MODE_AUTOMATIC: Cogl will try to automatically
+ *   decide which of the above two to use. For cogl_rectangle(), it
+ *   will use repeat mode if any of the texture coordinates are
+ *   outside the range 0→1, otherwise it will use clamp to edge. For
+ *   cogl_polygon() it will always use repeat mode. For
+ *   cogl_vertex_buffer_draw() it will use repeat mode except for
+ *   layers that have point sprite coordinate generation enabled. This
+ *   is the default value.
+ *
+ * The wrap mode specifies what happens when texture coordinates
+ * outside the range 0→1 are used. Note that if the filter mode is
+ * anything but %COGL_MATERIAL_FILTER_NEAREST then texels outside the
+ * range 0→1 might be used even when the coordinate is exactly 0 or 1
+ * because OpenGL will try to sample neighbouring pixels. For example
+ * if you are trying to render the full texture then you may get
+ * artifacts around the edges when the pixels from the other side are
+ * merged in if the wrap mode is set to repeat.
+ *
+ * Since: 1.4
+ */
+/* GL_ALWAYS is just used here as a value that is known not to clash
+ * with any valid GL wrap modes
+ *
+ * XXX: keep the values in sync with the CoglMaterialWrapModeInternal
+ * enum so no conversion is actually needed.
+ */
+typedef enum {
+  COGL_MATERIAL_WRAP_MODE_REPEAT = GL_REPEAT,
+  COGL_MATERIAL_WRAP_MODE_CLAMP_TO_EDGE = GL_CLAMP_TO_EDGE,
+  COGL_MATERIAL_WRAP_MODE_AUTOMATIC = GL_ALWAYS
+} CoglMaterialWrapMode;
+
 /**
  * cogl_material_new:
  *
  * Allocates and initializes a blank white material
  *
- * Return value: a handle to the new material
+ * Return value: a pointer to a new #CoglMaterial
  */
-CoglHandle
+CoglMaterial *
 cogl_material_new (void);
 
 /**
  * cogl_material_copy:
- * @source: the handle for the material to copy
+ * @source: a #CoglMaterial object to copy
  *
- * Creates a new material handle with the configuration copied from the
+ * Creates a new material with the configuration copied from the
  * source material.
  *
  * We would strongly advise developers to always aim to use
@@ -102,39 +147,39 @@ cogl_material_new (void);
  * keep track of a materials ancestry which we may use to help minimize GPU
  * state changes.
  *
- * Returns: a handle to the new material
+ * Returns: a pointer to the newly allocated #CoglMaterial
  *
  * Since: 1.2
  */
-CoglHandle
-cogl_material_copy (CoglHandle source);
+CoglMaterial *
+cogl_material_copy (CoglMaterial *source);
 
 #ifndef COGL_DISABLE_DEPRECATED
 
 /**
  * cogl_material_ref:
- * @handle: a @CoglHandle.
+ * @material: a #CoglMaterial object.
  *
- * Increment the reference count for a cogl material.
+ * Increment the reference count for a #CoglMaterial.
  *
- * Return value: the @handle.
+ * Return value: the @material.
  *
  * Since: 1.0
  *
- * Deprecated: 1.2: Use cogl_handle_ref() instead
+ * Deprecated: 1.2: Use cogl_object_ref() instead
  */
 CoglHandle
 cogl_material_ref (CoglHandle handle) G_GNUC_DEPRECATED;
 
 /**
  * cogl_material_unref:
- * @handle: a @CoglHandle.
+ * @material: a #CoglMaterial object.
  *
- * Decrement the reference count for a cogl material.
+ * Decrement the reference count for a #CoglMaterial.
  *
  * Since: 1.0
  *
- * Deprecated: 1.2: Use cogl_handle_unref() instead
+ * Deprecated: 1.2: Use cogl_object_unref() instead
  */
 void
 cogl_material_unref (CoglHandle handle) G_GNUC_DEPRECATED;
@@ -155,7 +200,7 @@ cogl_is_material (CoglHandle handle);
 
 /**
  * cogl_material_set_color:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @color: The components of the color
  *
  * Sets the basic color of the material, used when no lighting is enabled.
@@ -170,12 +215,12 @@ cogl_is_material (CoglHandle handle);
  * Since: 1.0
  */
 void
-cogl_material_set_color (CoglHandle       material,
+cogl_material_set_color (CoglMaterial    *material,
                          const CoglColor *color);
 
 /**
  * cogl_material_set_color4ub:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @red: The red component
  * @green: The green component
  * @blue: The blue component
@@ -188,15 +233,15 @@ cogl_material_set_color (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_set_color4ub (CoglHandle material,
-			    guint8     red,
-                            guint8     green,
-                            guint8     blue,
-                            guint8     alpha);
+cogl_material_set_color4ub (CoglMaterial *material,
+			    guint8        red,
+                            guint8        green,
+                            guint8        blue,
+                            guint8        alpha);
 
 /**
  * cogl_material_set_color4f:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @red: The red component
  * @green: The green component
  * @blue: The blue component
@@ -209,15 +254,15 @@ cogl_material_set_color4ub (CoglHandle material,
  * Since: 1.0
  */
 void
-cogl_material_set_color4f (CoglHandle material,
-                           float      red,
-                           float      green,
-                           float      blue,
-                           float      alpha);
+cogl_material_set_color4f (CoglMaterial *material,
+                           float         red,
+                           float         green,
+                           float         blue,
+                           float         alpha);
 
 /**
  * cogl_material_get_color:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @color: (out): The location to store the color
  *
  * Retrieves the current material color.
@@ -225,12 +270,12 @@ cogl_material_set_color4f (CoglHandle material,
  * Since: 1.0
  */
 void
-cogl_material_get_color (CoglHandle  material,
-                         CoglColor  *color);
+cogl_material_get_color (CoglMaterial *material,
+                         CoglColor    *color);
 
 /**
  * cogl_material_set_ambient:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @ambient: The components of the desired ambient color
  *
  * Sets the material's ambient color, in the standard OpenGL lighting
@@ -245,12 +290,12 @@ cogl_material_get_color (CoglHandle  material,
  * Since: 1.0
  */
 void
-cogl_material_set_ambient (CoglHandle       material,
+cogl_material_set_ambient (CoglMaterial    *material,
 			   const CoglColor *ambient);
 
 /**
  * cogl_material_get_ambient:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @ambient: The location to store the ambient color
  *
  * Retrieves the current ambient color for @material
@@ -258,12 +303,12 @@ cogl_material_set_ambient (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_get_ambient (CoglHandle  material,
-                           CoglColor  *ambient);
+cogl_material_get_ambient (CoglMaterial *material,
+                           CoglColor    *ambient);
 
 /**
  * cogl_material_set_diffuse:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @diffuse: The components of the desired diffuse color
  *
  * Sets the material's diffuse color, in the standard OpenGL lighting
@@ -275,12 +320,12 @@ cogl_material_get_ambient (CoglHandle  material,
  * Since: 1.0
  */
 void
-cogl_material_set_diffuse (CoglHandle       material,
+cogl_material_set_diffuse (CoglMaterial    *material,
 			   const CoglColor *diffuse);
 
 /**
  * cogl_material_get_diffuse:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @diffuse: The location to store the diffuse color
  *
  * Retrieves the current diffuse color for @material
@@ -288,12 +333,12 @@ cogl_material_set_diffuse (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_get_diffuse (CoglHandle  material,
-                           CoglColor  *diffuse);
+cogl_material_get_diffuse (CoglMaterial *material,
+                           CoglColor    *diffuse);
 
 /**
  * cogl_material_set_ambient_and_diffuse:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @color: The components of the desired ambient and diffuse colors
  *
  * Conveniently sets the diffuse and ambient color of @material at the same
@@ -306,12 +351,12 @@ cogl_material_get_diffuse (CoglHandle  material,
  * Since: 1.0
  */
 void
-cogl_material_set_ambient_and_diffuse (CoglHandle       material,
+cogl_material_set_ambient_and_diffuse (CoglMaterial    *material,
 				       const CoglColor *color);
 
 /**
  * cogl_material_set_specular:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @specular: The components of the desired specular color
  *
  * Sets the material's specular color, in the standard OpenGL lighting
@@ -323,12 +368,12 @@ cogl_material_set_ambient_and_diffuse (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_set_specular (CoglHandle       material,
+cogl_material_set_specular (CoglMaterial    *material,
 			    const CoglColor *specular);
 
 /**
  * cogl_material_get_specular:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @specular: The location to store the specular color
  *
  * Retrieves the materials current specular color.
@@ -336,30 +381,29 @@ cogl_material_set_specular (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_get_specular (CoglHandle  material,
-                            CoglColor  *specular);
+cogl_material_get_specular (CoglMaterial *material,
+                            CoglColor    *specular);
 
 /**
  * cogl_material_set_shininess:
- * @material: A CoglMaterial object
- * @shininess: The desired shininess; must be >= 0.0
+ * @material: A #CoglMaterial object
+ * @shininess: The desired shininess; range: [0.0, 1.0]
  *
- * Sets the shininess of the material, in the standard OpenGL lighting
- * model, which determines the size of the specular highlights. A
- * higher @shininess will produce smaller highlights which makes the
- * object appear more shiny.
+ * Sets the materials shininess, in the standard OpenGL lighting model,
+ * which determines how specular highlights are calculated. A higher
+ * @shininess will produce smaller brigher highlights.
  *
  * The default value is 0.0
  *
  * Since: 1.0
  */
 void
-cogl_material_set_shininess (CoglHandle material,
-			     float      shininess);
+cogl_material_set_shininess (CoglMaterial *material,
+			     float         shininess);
 
 /**
  * cogl_material_get_shininess:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  *
  * Retrieves the materials current emission color.
  *
@@ -368,11 +412,11 @@ cogl_material_set_shininess (CoglHandle material,
  * Since: 1.0
  */
 float
-cogl_material_get_shininess (CoglHandle material);
+cogl_material_get_shininess (CoglMaterial *material);
 
 /**
  * cogl_material_set_emission:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @emission: The components of the desired emissive color
  *
  * Sets the material's emissive color, in the standard OpenGL lighting
@@ -384,12 +428,12 @@ cogl_material_get_shininess (CoglHandle material);
  * Since: 1.0
  */
 void
-cogl_material_set_emission (CoglHandle       material,
+cogl_material_set_emission (CoglMaterial    *material,
 			    const CoglColor *emission);
 
 /**
  * cogl_material_get_emission:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @emission: The location to store the emission color
  *
  * Retrieves the materials current emission color.
@@ -397,8 +441,8 @@ cogl_material_set_emission (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_get_emission (CoglHandle material,
-                            CoglColor *emission);
+cogl_material_get_emission (CoglMaterial *material,
+                            CoglColor    *emission);
 
 /**
  * CoglMaterialAlphaFunc:
@@ -435,7 +479,7 @@ typedef enum {
 
 /**
  * cogl_material_set_alpha_test_function:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @alpha_func: A @CoglMaterialAlphaFunc constant
  * @alpha_reference: A reference point that the chosen alpha function uses
  *   to compare incoming fragments to.
@@ -451,13 +495,13 @@ typedef enum {
  * Since: 1.0
  */
 void
-cogl_material_set_alpha_test_function (CoglHandle            material,
+cogl_material_set_alpha_test_function (CoglMaterial         *material,
 				       CoglMaterialAlphaFunc alpha_func,
 				       float                 alpha_reference);
 
 /**
  * cogl_material_set_blend:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @blend_string: A <link linkend="cogl-Blend-Strings">Cogl blend string</link>
  *   describing the desired blend function.
  * @error: return location for a #GError that may report lack of driver
@@ -540,13 +584,13 @@ cogl_material_set_alpha_test_function (CoglHandle            material,
  * Since: 1.0
  */
 gboolean
-cogl_material_set_blend (CoglHandle  material,
-                         const char *blend_string,
-                         GError    **error);
+cogl_material_set_blend (CoglMaterial *material,
+                         const char   *blend_string,
+                         GError      **error);
 
 /**
  * cogl_material_set_blend_constant:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @constant_color: The constant color you want
  *
  * When blending is setup to reference a CONSTANT blend factor then
@@ -555,12 +599,45 @@ cogl_material_set_blend (CoglHandle  material,
  * Since: 1.0
  */
 void
-cogl_material_set_blend_constant (CoglHandle material,
-                                  CoglColor *constant_color);
+cogl_material_set_blend_constant (CoglMaterial *material,
+                                  const CoglColor *constant_color);
+
+/**
+ * cogl_material_set_point_size:
+ * @material: a #CoglHandle to a material.
+ * @size: the new point size.
+ *
+ * Changes the size of points drawn when %COGL_VERTICES_MODE_POINTS is
+ * used with the vertex buffer API. Note that typically the GPU will
+ * only support a limited minimum and maximum range of point sizes. If
+ * the chosen point size is outside that range then the nearest value
+ * within that range will be used instead. The size of a point is in
+ * screen space so it will be the same regardless of any
+ * transformations. The default point size is 1.0.
+ *
+ * Since: 1.4
+ */
+void
+cogl_material_set_point_size (CoglHandle material,
+                              float      point_size);
+
+/**
+ * cogl_material_get_point_size:
+ * @material: a #CoglHandle to a material.
+ *
+ * Get the size of points drawn when %COGL_VERTICES_MODE_POINTS is
+ * used with the vertex buffer API.
+ *
+ * Return value: the point size of the material.
+ *
+ * Since: 1.4
+ */
+float
+cogl_material_get_point_size (CoglHandle  material);
 
 /**
  * cogl_material_set_layer:
- * @material: A #CoglHandle for a material object
+ * @material: A #CoglMaterial object
  * @layer_index: the index of the layer
  * @texture: a #CoglHandle for the layer object
  *
@@ -578,25 +655,25 @@ cogl_material_set_blend_constant (CoglHandle material,
  * Since: 1.0
  */
 void
-cogl_material_set_layer (CoglHandle material,
-			 int        layer_index,
-			 CoglHandle texture);
+cogl_material_set_layer (CoglMaterial *material,
+			 int           layer_index,
+			 CoglHandle    texture);
 
 /**
  * cogl_material_remove_layer:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @layer_index: Specifies the layer you want to remove
  *
  * This function removes a layer from your material
  */
 void
-cogl_material_remove_layer (CoglHandle material,
-			    int        layer_index);
+cogl_material_remove_layer (CoglMaterial *material,
+			    int           layer_index);
 
 
 /**
  * cogl_material_set_layer_combine:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @layer_index: Specifies the layer you want define a combine function for
  * @blend_string: A <link linkend="cogl-Blend-Strings">Cogl blend string</link>
  *    describing the desired texture combine function.
@@ -687,14 +764,14 @@ cogl_material_remove_layer (CoglHandle material,
  * Since: 1.0
  */
 gboolean
-cogl_material_set_layer_combine (CoglHandle   material,
-				 int          layer_index,
-				 const char  *blend_string,
-                                 GError     **error);
+cogl_material_set_layer_combine (CoglMaterial *material,
+				 int           layer_index,
+				 const char   *blend_string,
+                                 GError      **error);
 
 /**
  * cogl_material_set_layer_combine_constant:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @layer_index: Specifies the layer you want to specify a constant used
  *               for texture combining
  * @constant: The constant color you want
@@ -705,13 +782,13 @@ cogl_material_set_layer_combine (CoglHandle   material,
  * Since: 1.0
  */
 void
-cogl_material_set_layer_combine_constant (CoglHandle  material,
-                                          int         layer_index,
-                                          CoglColor  *constant);
+cogl_material_set_layer_combine_constant (CoglMaterial    *material,
+                                          int              layer_index,
+                                          const CoglColor *constant);
 
 /**
  * cogl_material_set_layer_matrix:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @layer_index: the index for the layer inside @material
  * @matrix: the transformation matrix for the layer
  *
@@ -719,28 +796,37 @@ cogl_material_set_layer_combine_constant (CoglHandle  material,
  * and rotate a single layer of a material used to fill your geometry.
  */
 void
-cogl_material_set_layer_matrix (CoglHandle  material,
-				int         layer_index,
-				CoglMatrix *matrix);
+cogl_material_set_layer_matrix (CoglMaterial     *material,
+				int               layer_index,
+				const CoglMatrix *matrix);
 
 /**
  * cogl_material_get_layers:
- * @material: a #CoglHandle for a material
+ * @material: A #CoglMaterial object
  *
- * This function lets you access a materials internal list of layers
+ * This function lets you access a material's internal list of layers
  * for iteration.
  *
- * Return value: (element-type Handle) (transfer none): A list of
- *   #CoglHandle<!-- -->'s that can be passed to the  cogl_material_layer_*
- *   functions. The list is owned by COGL and it  should not be modified or
- *   freed
+ * <note>You should avoid using this API if possible since it was only
+ * made public by mistake and will be deprecated when we have
+ * suitable alternative.</note>
+ *
+ * <note>It's important to understand that the list returned may not
+ * remain valid if you modify the material or any of the layers in any
+ * way and so you would have to re-get the list in that
+ * situation.</note>
+ *
+ * Return value: (element-type CoglMaterialLayer) (transfer none): A
+ *    list of #CoglMaterialLayer<!-- -->'s that can be passed to the
+ *    cogl_material_layer_* functions. The list is owned by Cogl and it
+ *    should not be modified or freed
  */
 G_CONST_RETURN GList *
-cogl_material_get_layers (CoglHandle material);
+cogl_material_get_layers (CoglMaterial *material);
 
 /**
  * cogl_material_get_n_layers:
- * @material: a #CoglHandle for a material
+ * @material: A #CoglMaterial object
  *
  * Retrieves the number of layers defined for the given @material
  *
@@ -749,7 +835,7 @@ cogl_material_get_layers (CoglHandle material);
  * Since: 1.0
  */
 int
-cogl_material_get_n_layers (CoglHandle material);
+cogl_material_get_n_layers (CoglMaterial *material);
 
 /**
  * CoglMaterialLayerType:
@@ -768,7 +854,7 @@ typedef enum {
 
 /**
  * cogl_material_layer_get_type:
- * @layer: A #CoglHandle for a material layer
+ * @layer: A #CoglMaterialLayer object
  *
  * Retrieves the type of the layer
  *
@@ -780,11 +866,11 @@ typedef enum {
  * Return value: the type of the layer
  */
 CoglMaterialLayerType
-cogl_material_layer_get_type (CoglHandle layer);
+cogl_material_layer_get_type (CoglMaterialLayer *layer);
 
 /**
  * cogl_material_layer_get_texture:
- * @layer: A #CoglHandle for a material layer
+ * @layer: A #CoglMaterialLayer object
  *
  * Extracts a texture handle for a specific layer.
  *
@@ -797,7 +883,7 @@ cogl_material_layer_get_type (CoglHandle layer);
  * Return value: a #CoglHandle for the texture inside the layer
  */
 CoglHandle
-cogl_material_layer_get_texture (CoglHandle layer);
+cogl_material_layer_get_texture (CoglMaterialLayer *layer);
 
 /**
  * cogl_material_layer_get_min_filter:
@@ -808,22 +894,22 @@ cogl_material_layer_get_texture (CoglHandle layer);
  * Return value: the current downscaling filter
  */
 CoglMaterialFilter
-cogl_material_layer_get_min_filter (CoglHandle layer);
+cogl_material_layer_get_min_filter (CoglMaterialLayer *layer);
 
 /**
  * cogl_material_layer_get_mag_filter:
- * @layer: a #CoglHandle for a material layer
+ * @layer: A #CoglMaterialLayer object
  *
  * Queries the currently set downscaling filter for a material later
  *
  * Return value: the current downscaling filter
  */
 CoglMaterialFilter
-cogl_material_layer_get_mag_filter (CoglHandle layer);
+cogl_material_layer_get_mag_filter (CoglMaterialLayer *layer);
 
 /**
  * cogl_material_set_layer_filters:
- * @material: a #CoglHandle to a material.
+ * @material: A #CoglMaterial object
  * @layer_index: the layer number to change.
  * @min_filter: the filter used when scaling a texture down.
  * @mag_filter: the filter used when magnifying a texture.
@@ -832,11 +918,343 @@ cogl_material_layer_get_mag_filter (CoglHandle layer);
  * drawn at other scales than 100%.
  */
 void
-cogl_material_set_layer_filters (CoglHandle         material,
+cogl_material_set_layer_filters (CoglMaterial      *material,
                                  int                layer_index,
                                  CoglMaterialFilter min_filter,
                                  CoglMaterialFilter mag_filter);
 
+/**
+ * cogl_material_set_layer_point_sprite_coords_enabled:
+ * @material: a #CoglHandle to a material.
+ * @layer_index: the layer number to change.
+ * @enable: whether to enable point sprite coord generation.
+ * @error: A return location for a GError, or NULL to ignore errors.
+ *
+ * When rendering points, if @enable is %TRUE then the texture
+ * coordinates for this layer will be replaced with coordinates that
+ * vary from 0.0 to 1.0 across the primitive. The top left of the
+ * point will have the coordinates 0.0,0.0 and the bottom right will
+ * have 1.0,1.0. If @enable is %FALSE then the coordinates will be
+ * fixed for the entire point.
+ *
+ * This function will only work if %COGL_FEATURE_POINT_SPRITE is
+ * available. If the feature is not available then the function will
+ * return %FALSE and set @error.
+ *
+ * Return value: %TRUE if the function succeeds, %FALSE otherwise.
+ * Since: 1.4
+ */
+gboolean
+cogl_material_set_layer_point_sprite_coords_enabled (CoglMaterial *material,
+                                                     int           layer_index,
+                                                     gboolean      enable,
+                                                     GError      **error);
+
+/**
+ * cogl_material_get_layer_point_sprite_coords_enabled:
+ * @material: a #CoglHandle to a material.
+ * @layer_index: the layer number to check.
+ *
+ * Gets whether point sprite coordinate generation is enabled for this
+ * texture layer.
+ *
+ * Return value: whether the texture coordinates will be replaced with
+ * point sprite coordinates.
+ *
+ * Since: 1.4
+ */
+gboolean
+cogl_material_get_layer_point_sprite_coords_enabled (CoglMaterial *material,
+                                                     int           layer_index);
+
+/**
+ * cogl_material_set_layer_wrap_mode_s:
+ * @material: A #CoglMaterial object
+ * @layer_index: the layer number to change.
+ * @mode: the new wrap mode
+ *
+ * Sets the wrap mode for the 's' coordinate of texture lookups on this layer.
+ *
+ * Since: 1.4
+ */
+void
+cogl_material_set_layer_wrap_mode_s (CoglMaterial        *material,
+                                     int                  layer_index,
+                                     CoglMaterialWrapMode mode);
+
+/**
+ * cogl_material_set_layer_wrap_mode_t:
+ * @material: A #CoglMaterial object
+ * @layer_index: the layer number to change.
+ * @mode: the new wrap mode
+ *
+ * Sets the wrap mode for the 't' coordinate of texture lookups on this layer.
+ *
+ * Since: 1.4
+ */
+void
+cogl_material_set_layer_wrap_mode_t (CoglMaterial        *material,
+                                     int                  layer_index,
+                                     CoglMaterialWrapMode mode);
+
+/**
+ * cogl_material_set_layer_wrap_mode:
+ * @material: A #CoglMaterial object
+ * @layer_index: the layer number to change.
+ * @mode: the new wrap mode
+ *
+ * Sets the wrap mode for both coordinates of texture lookups on this
+ * layer. This is equivalent to calling
+ * cogl_material_set_layer_wrap_mode_s() and
+ * cogl_material_set_layer_wrap_mode_t() separately.
+ *
+ * Since: 1.4
+ */
+void
+cogl_material_set_layer_wrap_mode (CoglMaterial        *material,
+                                   int                  layer_index,
+                                   CoglMaterialWrapMode mode);
+
+/**
+ * cogl_material_layer_get_wrap_mode_s:
+ * @layer: A #CoglMaterialLayer object
+ *
+ * Gets the wrap mode for the 's' coordinate of texture lookups on this layer.
+ *
+ * Return value: the wrap mode value for the s coordinate.
+ *
+ * Since: 1.4
+ */
+CoglMaterialWrapMode
+cogl_material_layer_get_wrap_mode_s (CoglMaterialLayer *layer);
+
+/**
+ * cogl_material_layer_get_wrap_mode_t:
+ * @layer: A #CoglMaterialLayer object
+ *
+ * Gets the wrap mode for the 't' coordinate of texture lookups on this layer.
+ *
+ * Return value: the wrap mode value for the t coordinate.
+ *
+ * Since: 1.4
+ */
+CoglMaterialWrapMode
+cogl_material_layer_get_wrap_mode_t (CoglMaterialLayer *layer);
+
+
+/* XXX: should this be CoglMaterialDepthTestFunction?
+ * It makes it very verbose but would be consistent with
+ * CoglMaterialWrapMode */
+
+/**
+ * CoglDepthTestFunction:
+ * @COGL_DEPTH_TEST_FUNCTION_NEVER: Never passes.
+ * @COGL_DEPTH_TEST_FUNCTION_LESS: Passes if the fragment's depth
+ * value is less than the value currently in the depth buffer.
+ * @COGL_DEPTH_TEST_FUNCTION_EQUAL: Passes if the fragment's depth
+ * value is equal to the value currently in the depth buffer.
+ * @COGL_DEPTH_TEST_FUNCTION_LEQUAL: Passes if the fragment's depth
+ * value is less or equal to the value currently in the depth buffer.
+ * @COGL_DEPTH_TEST_FUNCTION_GREATER: Passes if the fragment's depth
+ * value is greater than the value currently in the depth buffer.
+ * @COGL_DEPTH_TEST_FUNCTION_NOTEQUAL: Passes if the fragment's depth
+ * value is not equal to the value currently in the depth buffer.
+ * @COGL_DEPTH_TEST_FUNCTION_GEQUAL: Passes if the fragment's depth
+ * value greater than or equal to the value currently in the depth buffer.
+ * @COGL_DEPTH_TEST_FUNCTION_ALWAYS: Always passes.
+ *
+ * When using depth testing one of these functions is used to compare
+ * the depth of an incoming fragment against the depth value currently
+ * stored in the depth buffer. The function is changed using
+ * cogl_material_set_depth_test_function().
+ *
+ * The test is only done when depth testing is explicitly enabled. (See
+ * cogl_material_set_depth_test_enabled())
+ */
+typedef enum
+{
+  COGL_DEPTH_TEST_FUNCTION_NEVER    = GL_NEVER,
+  COGL_DEPTH_TEST_FUNCTION_LESS     = GL_LESS,
+  COGL_DEPTH_TEST_FUNCTION_EQUAL    = GL_EQUAL,
+  COGL_DEPTH_TEST_FUNCTION_LEQUAL   = GL_LEQUAL,
+  COGL_DEPTH_TEST_FUNCTION_GREATER  = GL_GREATER,
+  COGL_DEPTH_TEST_FUNCTION_NOTEQUAL = GL_NOTEQUAL,
+  COGL_DEPTH_TEST_FUNCTION_GEQUAL   = GL_GEQUAL,
+  COGL_DEPTH_TEST_FUNCTION_ALWAYS   = GL_ALWAYS
+} CoglDepthTestFunction;
+/* XXX: to avoid having to split this into a separate include that can
+ * in #included internally without needing the
+ * COGL_ENABLE_EXPERIMENTAL_API define this isn't guarded. It's still
+ * considered experimental but it's guarded instead by the fact that
+ * there's no corresponding API. */
+
+#ifdef COGL_ENABLE_EXPERIMENTAL_API
+
+/**
+ * cogl_material_set_depth_test_enabled:
+ * @material: A #CoglMaterial object
+ * @enable: The enable state you want
+ *
+ * Enables or disables depth testing according to the value of
+ * @enable.
+ *
+ * If depth testing is enable then the #CoglDepthTestFunction set
+ * using cogl_material_set_depth_test_function() us used to evaluate
+ * the depth value of incoming fragments against the corresponding
+ * value stored in the current depth buffer, and if the test passes
+ * then the fragments depth value is used to update the depth buffer.
+ * (unless you have disabled depth writing via
+ * cogl_material_set_depth_writing_enabled ())
+ *
+ * By default depth testing is disabled.
+ *
+ * Since: 1.4
+ * Stability: Unstable
+ */
+void
+cogl_material_set_depth_test_enabled (CoglMaterial *material,
+                                      gboolean enable);
+
+/**
+ * cogl_material_get_depth_test_enabled:
+ * @material: A #CoglMaterial object
+ *
+ * Gets the current depth test enabled state as previously set by
+ * cogl_material_set_depth_test_enabled().
+ *
+ * Returns: The material's current depth test enabled state.
+ * Since: 1.4
+ * Stability: Unstable
+ */
+gboolean
+cogl_material_get_depth_test_enabled (CoglMaterial *material);
+
+/**
+ * cogl_material_set_depth_writing_enabled:
+ * @material: A #CoglMaterial object
+ * @enable: The enable state you want
+ *
+ * Enables or disables depth buffer writing according to the value of
+ * @enable. Normally when depth testing is enabled and the comparison
+ * between a fragment's depth value and the corresponding depth buffer
+ * value passes then the fragment's depth is written to the depth
+ * buffer unless writing is disabled here.
+ *
+ * By default depth writing is enabled
+ *
+ * Since: 1.4
+ * Stability: Unstable
+ */
+void
+cogl_material_set_depth_writing_enabled (CoglMaterial *material,
+                                         gboolean enable);
+
+/**
+ * cogl_material_get_depth_writing_enabled:
+ * @material: A #CoglMaterial object
+ *
+ * Gets the depth writing enable state as set by the corresponding
+ * cogl_material_set_depth_writing_enabled.
+ *
+ * Returns: The current depth writing enable state
+ * Since: 1.4
+ * Stability: Unstable
+ */
+gboolean
+cogl_material_get_depth_writing_enabled (CoglMaterial *material);
+
+/**
+ * cogl_material_set_depth_test_function:
+ * @material: A #CoglMaterial object
+ * @function: The #CoglDepthTestFunction to set
+ *
+ * Sets the #CoglDepthTestFunction used to compare the depth value of
+ * an incoming fragment against the corresponding value in the current
+ * depth buffer.
+ *
+ * Since: 1.4
+ * Stability: Unstable
+ */
+void
+cogl_material_set_depth_test_function (CoglMaterial *material,
+                                       CoglDepthTestFunction function);
+
+/**
+ * cogl_material_get_depth_test_function:
+ * @material: A #CoglMaterial object
+ *
+ * Gets the current depth test enable state as previously set via
+ * cogl_material_set_depth_test_enabled().
+ *
+ * Returns: The current depth test enable state.
+ * Since: 1.4
+ * Stability: Unstable
+ */
+CoglDepthTestFunction
+cogl_material_get_depth_test_function (CoglMaterial *material);
+
+/**
+ * cogl_material_set_depth_range:
+ * @material: A #CoglMaterial object
+ * @near_val: The near component of the desired depth range which will be
+ * clamped to the range [0, 1]
+ * @far_val: The far component of the desired depth range which will be
+ * clamped to the range [0, 1]
+ * @error: location to store an error of type #CoglError
+ *
+ * Sets the range to map depth values in normalized device coordinates
+ * to before writing out to a depth buffer.
+ *
+ * After your geometry has be transformed, clipped and had perspective
+ * division applied placing it in normalized device
+ * coordinates all depth values between the near and far z clipping
+ * planes are in the range -1 to 1. Before writing any depth value to
+ * the depth buffer though the value is mapped into the range [0, 1].
+ *
+ * With this function you can change the range which depth values are
+ * mapped too although the range must still lye within the range [0,
+ * 1].
+ *
+ * If your driver does not support this feature (for example you are
+ * using GLES 1 drivers) then this will return %FALSE and set an error
+ * if @error isn't NULL. You can check ahead of time for the
+ * %COGL_FEATURE_DEPTH_RANGE feature with cogl_features_available() to
+ * know if this function will succeed.
+ *
+ * By default normalized device coordinate depth values are mapped to
+ * the full range of depth buffer values, [0, 1].
+ *
+ * Returns: %TRUE if driver support is available else %FALSE.
+ *
+ * Since: 1.4
+ * Stability: Unstable
+ */
+gboolean
+cogl_material_set_depth_range (CoglMaterial *material,
+                               float near_val,
+                               float far_val,
+                               GError **error);
+
+/**
+ * cogl_material_get_depth_range_mapping:
+ * @material: A #CoglMaterial object
+ * @near_val: A pointer to store the near component of the depth range
+ * @far_val: A pointer to store the far component of the depth range
+ *
+ * Gets the current range to which normalized depth values are mapped
+ * before writing to the depth buffer. This corresponds to the range
+ * set with cogl_material_set_depth_range().
+ *
+ * Since: 1.4
+ * Stability: Unstable
+ */
+void
+cogl_material_get_depth_range (CoglMaterial *material,
+                               float *near_val,
+                               float *far_val);
+
+#endif /* COGL_ENABLE_EXPERIMENTAL_API */
+
 G_END_DECLS
 
 #endif /* __COGL_MATERIAL_H__ */
