@@ -3,7 +3,7 @@
  *
  * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Copyright (C) 2007,2008,2009 Intel Corporation.
+ * Copyright (C) 2007,2008,2009,2010 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,6 +41,9 @@
 #include "cogl-framebuffer-private.h"
 #include "cogl-matrix-private.h"
 #include "cogl-journal-private.h"
+#include "cogl-bitmap-private.h"
+#include "cogl-texture-private.h"
+#include "cogl-texture-driver.h"
 
 #if defined (HAVE_COGL_GLES2) || defined (HAVE_COGL_GLES)
 #include "cogl-gles2-wrapper.h"
@@ -54,7 +57,7 @@
 /* GL error to string conversion */
 static const struct {
   GLuint error_code;
-  const gchar *error_string;
+  const char *error_string;
 } gl_errors[] = {
   { GL_NO_ERROR,          "No error" },
   { GL_INVALID_ENUM,      "Invalid enumeration value" },
@@ -69,12 +72,12 @@ static const struct {
 #endif
 };
 
-static const guint n_gl_errors = G_N_ELEMENTS (gl_errors);
+static const unsigned int n_gl_errors = G_N_ELEMENTS (gl_errors);
 
-const gchar *
+const char *
 cogl_gl_error_to_string (GLenum error_code)
 {
-  gint i;
+  int i;
 
   for (i = 0; i < n_gl_errors; i++)
     {
@@ -87,7 +90,7 @@ cogl_gl_error_to_string (GLenum error_code)
 #endif /* COGL_GL_DEBUG */
 
 CoglFuncPtr
-cogl_get_proc_address (const gchar* name)
+cogl_get_proc_address (const char* name)
 {
   void *address;
   static GModule *module = NULL;
@@ -112,8 +115,40 @@ cogl_get_proc_address (const gchar* name)
   return NULL;
 }
 
+gboolean
+_cogl_check_extension (const char *name, const gchar *ext)
+{
+  char *end;
+  int name_len, n;
+
+  if (name == NULL || ext == NULL)
+    return FALSE;
+
+  end = (char*)(ext + strlen(ext));
+
+  name_len = strlen(name);
+
+  while (ext < end)
+    {
+      n = strcspn(ext, " ");
+
+      if ((name_len == n) && (!strncmp(name, ext, n)))
+	return TRUE;
+      ext += (n + 1);
+    }
+
+  return FALSE;
+}
+
+/* XXX: This has been deprecated as public API */
+gboolean
+cogl_check_extension (const char *name, const char *ext)
+{
+  return _cogl_check_extension (name, ext);
+}
+
 void
-cogl_clear (const CoglColor *color, gulong buffers)
+cogl_clear (const CoglColor *color, unsigned long buffers)
 {
   GLbitfield gl_buffers = 0;
 
@@ -154,7 +189,7 @@ cogl_clear (const CoglColor *color, gulong buffers)
       return;
     }
 
-  glClear (gl_buffers);
+  GE (glClear (gl_buffers));
 
   /* This is a debugging variable used to visually display the quad
      batches from the journal. It is reset here to increase the
@@ -169,11 +204,11 @@ cogl_clear (const CoglColor *color, gulong buffers)
   COGL_NOTE (DRAW, "Clear end");
 }
 
-static inline gboolean
-cogl_toggle_flag (CoglContext *ctx,
-		  gulong new_flags,
-		  gulong flag,
-		  GLenum gl_flag)
+static gboolean
+toggle_flag (CoglContext *ctx,
+	     unsigned long new_flags,
+	     unsigned long flag,
+	     GLenum gl_flag)
 {
   /* Toggles and caches a single enable flag on or off
    * by comparing to current state
@@ -196,11 +231,11 @@ cogl_toggle_flag (CoglContext *ctx,
   return FALSE;
 }
 
-static inline gboolean
-cogl_toggle_client_flag (CoglContext *ctx,
-			 gulong new_flags,
-			 gulong flag,
-			 GLenum gl_flag)
+static gboolean
+toggle_client_flag (CoglContext *ctx,
+		    unsigned long new_flags,
+		    unsigned long flag,
+		    GLenum gl_flag)
 {
   /* Toggles and caches a single client-side enable flag
    * on or off by comparing to current state
@@ -224,57 +259,56 @@ cogl_toggle_client_flag (CoglContext *ctx,
 }
 
 void
-cogl_enable (gulong flags)
+_cogl_enable (unsigned long flags)
 {
   /* This function essentially caches glEnable state() in the
    * hope of lessening number GL traffic.
   */
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  cogl_toggle_flag (ctx, flags,
-                    COGL_ENABLE_BLEND,
-                    GL_BLEND);
+  toggle_flag (ctx, flags,
+               COGL_ENABLE_BACKFACE_CULLING,
+               GL_CULL_FACE);
 
-  cogl_toggle_flag (ctx, flags,
-                    COGL_ENABLE_BACKFACE_CULLING,
-                    GL_CULL_FACE);
+  toggle_client_flag (ctx, flags,
+		      COGL_ENABLE_VERTEX_ARRAY,
+		      GL_VERTEX_ARRAY);
 
-  cogl_toggle_client_flag (ctx, flags,
-			   COGL_ENABLE_VERTEX_ARRAY,
-			   GL_VERTEX_ARRAY);
-
-  cogl_toggle_client_flag (ctx, flags,
-			   COGL_ENABLE_COLOR_ARRAY,
-			   GL_COLOR_ARRAY);
+  toggle_client_flag (ctx, flags,
+		      COGL_ENABLE_COLOR_ARRAY,
+		      GL_COLOR_ARRAY);
 }
 
-gulong
-cogl_get_enable ()
+unsigned long
+_cogl_get_enable (void)
 {
   _COGL_GET_CONTEXT (ctx, 0);
 
   return ctx->enable_flags;
 }
 
+/* XXX: This API has been deprecated */
 void
 cogl_set_depth_test_enabled (gboolean setting)
 {
-  /* Currently the journal can't track changes to depth state... */
-  _cogl_journal_flush ();
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  if (setting)
-    {
-      glEnable (GL_DEPTH_TEST);
-      glDepthFunc (GL_LEQUAL);
-    }
+  if (ctx->legacy_depth_test_enabled == setting)
+    return;
+
+  ctx->legacy_depth_test_enabled = setting;
+  if (ctx->legacy_depth_test_enabled)
+    ctx->legacy_state_set++;
   else
-    glDisable (GL_DEPTH_TEST);
+    ctx->legacy_state_set--;
 }
 
+/* XXX: This API has been deprecated */
 gboolean
 cogl_get_depth_test_enabled (void)
 {
-  return glIsEnabled (GL_DEPTH_TEST) == GL_TRUE ? TRUE : FALSE;
+  _COGL_GET_CONTEXT (ctx, FALSE);
+  return ctx->legacy_depth_test_enabled;
 }
 
 void
@@ -354,7 +388,7 @@ cogl_set_viewport (int x,
                    int width,
                    int height)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -370,15 +404,15 @@ cogl_set_viewport (int x,
 /* XXX: This should be deprecated, and we should expose a way to also
  * specify an x and y viewport offset */
 void
-cogl_viewport (guint width,
-	       guint height)
+cogl_viewport (unsigned int width,
+	       unsigned int height)
 {
   cogl_set_viewport (0, 0, width, height);
 }
 
 void
-_cogl_setup_viewport (guint width,
-                      guint height,
+_cogl_setup_viewport (unsigned int width,
+                      unsigned int height,
                       float fovy,
                       float aspect,
                       float z_near,
@@ -455,9 +489,15 @@ cogl_get_features (void)
 {
   _COGL_GET_CONTEXT (ctx, 0);
 
-  if (cogl_debug_flags & COGL_DEBUG_DISABLE_VBOS)
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_VBOS))
     ctx->feature_flags &= ~COGL_FEATURE_VBOS;
 
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_PBOS))
+    ctx->feature_flags &= ~COGL_FEATURE_PBOS;
+
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_GLSL))
+    ctx->feature_flags &= ~COGL_FEATURE_SHADERS_GLSL;
+
   return ctx->feature_flags;
 }
 
@@ -472,6 +512,20 @@ cogl_features_available (CoglFeatureFlags features)
   return (ctx->feature_flags & features) == features;
 }
 
+gboolean
+_cogl_features_available_private (CoglFeatureFlagsPrivate features)
+{
+  _COGL_GET_CONTEXT (ctx, 0);
+
+  if (!ctx->features_cached)
+    _cogl_features_init ();
+
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_ARBFP))
+    ctx->feature_flags_private &= ~COGL_FEATURE_PRIVATE_ARB_FP;
+
+  return (ctx->feature_flags_private & features) == features;
+}
+
 /* XXX: This function should either be replaced with one returning
  * integers, or removed/deprecated and make the
  * _cogl_framebuffer_get_viewport* functions public.
@@ -479,7 +533,7 @@ cogl_features_available (CoglFeatureFlags features)
 void
 cogl_get_viewport (float v[4])
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   int viewport[4];
   int i;
 
@@ -493,36 +547,26 @@ cogl_get_viewport (float v[4])
 }
 
 void
-cogl_get_bitmasks (gint *red,
-                   gint *green,
-                   gint *blue,
-                   gint *alpha)
+cogl_get_bitmasks (int *red,
+                   int *green,
+                   int *blue,
+                   int *alpha)
 {
-  GLint value;
+  CoglFramebuffer *framebuffer;
+
+  framebuffer = _cogl_get_framebuffer ();
 
   if (red)
-    {
-      GE( glGetIntegerv(GL_RED_BITS, &value) );
-      *red = value;
-    }
+    *red = _cogl_framebuffer_get_red_bits (framebuffer);
 
   if (green)
-    {
-      GE( glGetIntegerv(GL_GREEN_BITS, &value) );
-      *green = value;
-    }
+    *green = _cogl_framebuffer_get_green_bits (framebuffer);
 
   if (blue)
-    {
-      GE( glGetIntegerv(GL_BLUE_BITS, &value) );
-      *blue = value;
-    }
+    *blue = _cogl_framebuffer_get_blue_bits (framebuffer);
 
   if (alpha)
-    {
-      GE( glGetIntegerv(GL_ALPHA_BITS, &value ) );
-      *alpha = value;
-    }
+    *alpha = _cogl_framebuffer_get_alpha_bits (framebuffer);
 }
 
 void
@@ -532,53 +576,28 @@ cogl_set_fog (const CoglColor *fog_color,
               float            z_near,
               float            z_far)
 {
-  GLfloat fogColor[4];
-  GLenum gl_mode = GL_LINEAR;
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  /* The cogl journal doesn't currently track fog state changes */
-  _cogl_journal_flush ();
+  if (ctx->legacy_fog_state.enabled == FALSE)
+    ctx->legacy_state_set++;
 
-  fogColor[0] = cogl_color_get_red_float (fog_color);
-  fogColor[1] = cogl_color_get_green_float (fog_color);
-  fogColor[2] = cogl_color_get_blue_float (fog_color);
-  fogColor[3] = cogl_color_get_alpha_float (fog_color);
-
-  glEnable (GL_FOG);
-
-  glFogfv (GL_FOG_COLOR, fogColor);
-
-#if HAVE_COGL_GLES
-  switch (mode)
-    {
-    case COGL_FOG_MODE_LINEAR:
-      gl_mode = GL_LINEAR;
-      break;
-    case COGL_FOG_MODE_EXPONENTIAL:
-      gl_mode = GL_EXP;
-      break;
-    case COGL_FOG_MODE_EXPONENTIAL_SQUARED:
-      gl_mode = GL_EXP2;
-      break;
-    }
-#endif
-  /* TODO: support other modes for GLES2 */
-
-  /* NB: GLES doesn't have glFogi */
-  glFogf (GL_FOG_MODE, gl_mode);
-  glHint (GL_FOG_HINT, GL_NICEST);
-
-  glFogf (GL_FOG_DENSITY, (GLfloat) density);
-  glFogf (GL_FOG_START, (GLfloat) z_near);
-  glFogf (GL_FOG_END, (GLfloat) z_far);
+  ctx->legacy_fog_state.enabled = TRUE;
+  ctx->legacy_fog_state.color = *fog_color;
+  ctx->legacy_fog_state.mode = mode;
+  ctx->legacy_fog_state.density = density;
+  ctx->legacy_fog_state.z_near = z_near;
+  ctx->legacy_fog_state.z_far = z_far;
 }
 
 void
 cogl_disable_fog (void)
 {
-  /* Currently the journal can't track changes to fog state... */
-  _cogl_journal_flush ();
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  glDisable (GL_FOG);
+  if (ctx->legacy_fog_state.enabled == TRUE)
+    ctx->legacy_state_set--;
+
+  ctx->legacy_fog_state.enabled = FALSE;
 }
 
 void
@@ -596,22 +615,22 @@ cogl_read_pixels (int x,
                   CoglPixelFormat format,
                   guint8 *pixels)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   int        framebuffer_height;
-  int        rowstride = width * 4;
-  guint8    *temprow;
+  int        bpp;
+  CoglBitmap bmp;
+  GLenum     gl_intformat;
+  GLenum     gl_format;
+  GLenum     gl_type;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  g_return_if_fail (format == COGL_PIXEL_FORMAT_RGBA_8888);
   g_return_if_fail (source == COGL_READ_PIXELS_COLOR_BUFFER);
 
   /* make sure any batched primitives get emitted to the GL driver before
    * issuing our read pixels... */
   cogl_flush ();
 
-  temprow = g_alloca (rowstride * sizeof (guint8));
-
   framebuffer = _cogl_get_framebuffer ();
 
   _cogl_framebuffer_flush_state (framebuffer, 0);
@@ -627,47 +646,151 @@ cogl_read_pixels (int x,
   if (!cogl_is_offscreen (framebuffer))
     y = framebuffer_height - y - height;
 
-  /* Setup the pixel store parameters that may have been changed by
-     Cogl */
-  GE (glPixelStorei (GL_PACK_ALIGNMENT, 1));
-#ifdef HAVE_COGL_GL
-  GE (glPixelStorei (GL_PACK_ROW_LENGTH, 0));
-  GE (glPixelStorei (GL_PACK_SKIP_PIXELS, 0));
-  GE (glPixelStorei (GL_PACK_SKIP_ROWS, 0));
-#endif /* HAVE_COGL_GL */
+  /* Initialise the CoglBitmap */
+  bpp = _cogl_get_format_bpp (format);
+  bmp.format = format;
+  bmp.data = pixels;
+  bmp.width = width;
+  bmp.height = height;
+  bmp.rowstride = bpp * width;
 
-  GE (glReadPixels (x, y, width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixels));
+  if ((format & COGL_A_BIT))
+    {
+      /* FIXME: We are assuming glReadPixels will always give us
+         premultiplied data so we'll set the premult flag on the
+         bitmap format. This will usually be correct because the
+         result of the default blending operations for Cogl ends up
+         with premultiplied data in the framebuffer. However it is
+         possible for the framebuffer to be in whatever format
+         depending on what CoglMaterial is used to render to
+         it. Eventually we may want to add a way for an application to
+         inform Cogl that the framebuffer is not premultiplied in case
+         it is being used for some special purpose. */
+      bmp.format |= COGL_PREMULT_BIT;
+    }
+
+  _cogl_pixel_format_to_gl (format, &gl_intformat, &gl_format, &gl_type);
+
+  /* Under GLES only GL_RGBA with GL_UNSIGNED_BYTE as well as an
+     implementation specific format under
+     GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES and
+     GL_IMPLEMENTATION_COLOR_READ_TYPE_OES is supported. We could try
+     to be more clever and check if the requested type matches that
+     but we would need some reliable functions to convert from GL
+     types to Cogl types. For now, lets just always read in
+     GL_RGBA/GL_UNSIGNED_BYTE and convert if necessary */
+#ifndef COGL_HAS_GL
+  if (gl_format != GL_RGBA || gl_type != GL_UNSIGNED_BYTE)
+    {
+      CoglBitmap tmp_bmp, dst_bmp;
+
+      tmp_bmp.format = COGL_PIXEL_FORMAT_RGBA_8888_PRE;
+      tmp_bmp.data = g_malloc (width * height * 4);
+      tmp_bmp.width = width;
+      tmp_bmp.height = height;
+      tmp_bmp.rowstride = 4 * width;
+
+      _cogl_texture_driver_prep_gl_for_pixels_download (tmp_bmp.rowstride, 4);
+
+      GE( glReadPixels (x, y, width, height,
+                        GL_RGBA, GL_UNSIGNED_BYTE,
+                        tmp_bmp.data) );
+
+      /* CoglBitmap doesn't currently have a way to convert without
+         allocating its own buffer so we have to copy the data
+         again */
+      if (_cogl_bitmap_convert_format_and_premult (&tmp_bmp,
+                                                   &dst_bmp,
+                                                   bmp.format))
+        {
+          _cogl_bitmap_copy_subregion (&dst_bmp,
+                                       &bmp,
+                                       0, 0,
+                                       0, 0,
+                                       bmp.width, bmp.height);
+          g_free (dst_bmp.data);
+        }
+      else
+        {
+          /* FIXME: there's no way to report an error here so we'll
+             just have to leave the data initialised */
+        }
+
+      g_free (tmp_bmp.data);
+    }
+  else
+#endif
+    {
+      _cogl_texture_driver_prep_gl_for_pixels_download (bmp.rowstride, bpp);
+
+      GE( glReadPixels (x, y, width, height, gl_format, gl_type, pixels) );
+
+      /* Convert to the premult format specified by the caller
+         in-place. This will do nothing if the premult status is already
+         correct. */
+      _cogl_bitmap_convert_premult_status (&bmp, format);
+    }
 
   /* NB: All offscreen rendering is done upside down so there is no need
    * to flip in this case... */
-  if (cogl_is_offscreen (framebuffer))
-    return;
-
-  /* TODO: consider using the GL_MESA_pack_invert extension in the future
-   * to avoid this flip... */
-
-  /* vertically flip the buffer in-place */
-  for (y = 0; y < height / 2; y++)
+  if (!cogl_is_offscreen (framebuffer))
     {
-      if (y != height - y - 1) /* skip center row */
+      guint8 *temprow = g_alloca (bmp.rowstride * sizeof (guint8));
+
+      /* TODO: consider using the GL_MESA_pack_invert extension in the future
+       * to avoid this flip... */
+
+      /* vertically flip the buffer in-place */
+      for (y = 0; y < height / 2; y++)
         {
-          memcpy (temprow,
-                  pixels + y * rowstride, rowstride);
-          memcpy (pixels + y * rowstride,
-                  pixels + (height - y - 1) * rowstride, rowstride);
-          memcpy (pixels + (height - y - 1) * rowstride,
-                  temprow,
-                  rowstride);
+          if (y != height - y - 1) /* skip center row */
+            {
+              memcpy (temprow,
+                      pixels + y * bmp.rowstride, bmp.rowstride);
+              memcpy (pixels + y * bmp.rowstride,
+                      pixels + (height - y - 1) * bmp.rowstride, bmp.rowstride);
+              memcpy (pixels + (height - y - 1) * bmp.rowstride,
+                      temprow,
+                      bmp.rowstride);
+            }
         }
     }
 }
 
+static void
+_cogl_disable_other_texcoord_arrays_cb (int texcoord_array_num, gpointer data)
+{
+  CoglContext *ctx = data;
+
+  GE (glClientActiveTexture (GL_TEXTURE0 + texcoord_array_num));
+  GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
+}
+
+void
+_cogl_disable_other_texcoord_arrays (const CoglBitmask *mask)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  /* Set texcoord_arrays_to_disable to only contain the arrays we want
+     to disable */
+  _cogl_bitmask_clear_all (&ctx->texcoord_arrays_to_disable);
+  _cogl_bitmask_set_bits (&ctx->texcoord_arrays_to_disable,
+                          &ctx->texcoord_arrays_enabled);
+  _cogl_bitmask_clear_bits (&ctx->texcoord_arrays_to_disable, mask);
+
+  _cogl_bitmask_foreach (&ctx->texcoord_arrays_to_disable,
+                         _cogl_disable_other_texcoord_arrays_cb, ctx);
+
+  /* Update the mask of arrays that are enabled */
+  _cogl_bitmask_clear_bits (&ctx->texcoord_arrays_enabled,
+                            &ctx->texcoord_arrays_to_disable);
+  _cogl_bitmask_set_bits (&ctx->texcoord_arrays_enabled, mask);
+}
+
 void
 cogl_begin_gl (void)
 {
-  CoglMaterialFlushOptions options;
-  gulong enable_flags = 0;
-  int i;
+  unsigned long enable_flags = 0;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -701,26 +824,17 @@ cogl_begin_gl (void)
    * A user should instead call cogl_set_source_color4ub() before
    * cogl_begin_gl() to simplify the state flushed.
    */
-  options.flags = 0;
-  _cogl_material_flush_gl_state (ctx->source_material, &options);
-
-  /* FIXME: This api is a bit yukky, ideally it will be removed if we
-   * re-work the cogl_enable mechanism */
-  enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
+  _cogl_material_flush_gl_state (ctx->source_material, FALSE);
 
   if (ctx->enable_backface_culling)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
 
-  cogl_enable (enable_flags);
+  _cogl_enable (enable_flags);
   _cogl_flush_face_winding ();
 
   /* Disable all client texture coordinate arrays */
-  for (i = 0; i < ctx->n_texcoord_arrays_enabled; i++)
-    {
-      GE (glClientActiveTexture (GL_TEXTURE0 + i));
-      GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
-    }
-  ctx->n_texcoord_arrays_enabled = 0;
+  _cogl_bitmask_clear_all (&ctx->temp_bitmask);
+  _cogl_disable_other_texcoord_arrays (&ctx->temp_bitmask);
 }
 
 void
@@ -739,96 +853,6 @@ cogl_end_gl (void)
   ctx->in_begin_gl_block = FALSE;
 }
 
-static CoglTextureUnit *
-_cogl_texture_unit_new (int index_)
-{
-  CoglTextureUnit *unit = g_new0 (CoglTextureUnit, 1);
-  unit->matrix_stack = _cogl_matrix_stack_new ();
-  unit->index = index_;
-  return unit;
-}
-
-static void
-_cogl_texture_unit_free (CoglTextureUnit *unit)
-{
-  _cogl_matrix_stack_destroy (unit->matrix_stack);
-  g_free (unit);
-}
-
-CoglTextureUnit *
-_cogl_get_texture_unit (int index_)
-{
-  GList *l;
-  CoglTextureUnit *unit;
-
-  _COGL_GET_CONTEXT (ctx, NULL);
-
-  for (l = ctx->texture_units; l; l = l->next)
-    {
-      unit = l->data;
-
-      if (unit->index == index_)
-        return unit;
-
-      /* The units are always sorted, so at this point we know this unit
-       * doesn't exist */
-      if (unit->index > index_)
-        break;
-    }
-  /* NB: if we now insert a new layer before l, that will maintain order.
-   */
-
-  unit = _cogl_texture_unit_new (index_);
-
-  /* Note: see comment after for() loop above */
-  ctx->texture_units =
-    g_list_insert_before (ctx->texture_units, l, unit);
-
-  return unit;
-}
-
-void
-_cogl_destroy_texture_units (void)
-{
-  GList *l;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  for (l = ctx->texture_units; l; l = l->next)
-    _cogl_texture_unit_free (l->data);
-  g_list_free (ctx->texture_units);
-}
-
-/*
- * This is more complicated than that, another pass needs to be done when
- * cogl have a neat way of saying if we are using the fixed function pipeline
- * or not (for the GL case):
- * MAX_TEXTURE_UNITS: fixed function pipeline, a texture unit has both a
- *                    sampler and a set of texture coordinates
- * MAX_TEXTURE_IMAGE_UNITS: number of samplers one can use from a fragment
- *                          program/shader (ARBfp1.0 asm/GLSL)
- * MAX_VERTEX_TEXTURE_UNITS: number of samplers one can use from a vertex
- *                           program/shader (can be 0)
- * MAX_COMBINED_TEXTURE_IMAGE_UNITS: Maximum samplers one can use, counting both
- *                                   the vertex and fragment shaders
- *
- * If both the vertex shader and the fragment processing stage access the same
- * texture image unit, then that counts as using two texture image units
- * against the latter limit: http://www.opengl.org/sdk/docs/man/xhtml/glGet.xml
- *
- * Note that, for now, we use GL_MAX_TEXTURE_UNITS as we are exposing the
- * fixed function pipeline.
- */
-guint
-_cogl_get_max_texture_image_units (void)
-{
-  GLint nb_texture_image_units;
-
-  GE( glGetIntegerv(GL_MAX_TEXTURE_UNITS, &nb_texture_image_units) );
-
-  return nb_texture_image_units;
-}
-
 void
 cogl_push_matrix (void)
 {
@@ -869,6 +893,14 @@ cogl_rotate (float angle, float x, float y, float z)
   _cogl_matrix_stack_rotate (modelview_stack, angle, x, y, z);
 }
 
+void
+cogl_transform (const CoglMatrix *matrix)
+{
+  CoglMatrixStack *modelview_stack =
+    _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_multiply (modelview_stack, matrix);
+}
+
 void
 cogl_perspective (float fov_y,
 		  float aspect,
@@ -967,10 +999,10 @@ cogl_set_projection_matrix (CoglMatrix *matrix)
   _COGL_MATRIX_DEBUG_PRINT (matrix);
 }
 
-CoglClipStackState *
+CoglClipState *
 _cogl_get_clip_state (void)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
 
   framebuffer = _cogl_get_framebuffer ();
   return _cogl_framebuffer_get_clip_state (framebuffer);
@@ -981,3 +1013,107 @@ _cogl_driver_error_quark (void)
 {
   return g_quark_from_static_string ("cogl-driver-error-quark");
 }
+
+void
+cogl_set_source (CoglHandle material_handle)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  g_return_if_fail (cogl_is_material (material_handle));
+
+  if (ctx->source_material == material_handle)
+    return;
+
+  cogl_handle_ref (material_handle);
+
+  if (ctx->source_material)
+    cogl_handle_unref (ctx->source_material);
+
+  ctx->source_material = material_handle;
+}
+
+void
+cogl_set_source_texture (CoglHandle texture_handle)
+{
+  CoglColor white;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  g_return_if_fail (texture_handle != NULL);
+
+  cogl_material_set_layer (ctx->simple_material, 0, texture_handle);
+  cogl_color_set_from_4ub (&white, 0xff, 0xff, 0xff, 0xff);
+  cogl_material_set_color (ctx->simple_material, &white);
+  cogl_set_source (ctx->simple_material);
+}
+
+void
+cogl_set_source_color4ub (guint8 red,
+                          guint8 green,
+                          guint8 blue,
+                          guint8 alpha)
+{
+  CoglColor c = { 0, };
+
+  cogl_color_set_from_4ub (&c, red, green, blue, alpha);
+  cogl_set_source_color (&c);
+}
+
+void
+cogl_set_source_color4f (float red,
+                         float green,
+                         float blue,
+                         float alpha)
+{
+  CoglColor c = { 0, };
+
+  cogl_color_set_from_4f (&c, red, green, blue, alpha);
+  cogl_set_source_color (&c);
+}
+
+/* Scale from OpenGL normalized device coordinates (ranging from -1 to 1)
+ * to Cogl window/framebuffer coordinates (ranging from 0 to buffer-size) with
+ * (0,0) being top left. */
+#define VIEWPORT_TRANSFORM_X(x, vp_origin_x, vp_width) \
+    (  ( ((x) + 1.0) * ((vp_width) / 2.0) ) + (vp_origin_x)  )
+/* Note: for Y we first flip all coordinates around the X axis while in
+ * normalized device coodinates */
+#define VIEWPORT_TRANSFORM_Y(y, vp_origin_y, vp_height) \
+    (  ( ((-(y)) + 1.0) * ((vp_height) / 2.0) ) + (vp_origin_y)  )
+
+/* Transform a homogeneous vertex position from model space to Cogl
+ * window coordinates (with 0,0 being top left) */
+void
+_cogl_transform_point (const CoglMatrix *matrix_mv,
+                       const CoglMatrix *matrix_p,
+                       const float *viewport,
+                       float *x,
+                       float *y)
+{
+  float z = 0;
+  float w = 1;
+
+  /* Apply the modelview matrix transform */
+  cogl_matrix_transform_point (matrix_mv, x, y, &z, &w);
+
+  /* Apply the projection matrix transform */
+  cogl_matrix_transform_point (matrix_p, x, y, &z, &w);
+
+  /* Perform perspective division */
+  *x /= w;
+  *y /= w;
+
+  /* Apply viewport transform */
+  *x = VIEWPORT_TRANSFORM_X (*x, viewport[0], viewport[2]);
+  *y = VIEWPORT_TRANSFORM_Y (*y, viewport[1], viewport[3]);
+}
+
+#undef VIEWPORT_TRANSFORM_X
+#undef VIEWPORT_TRANSFORM_Y
+
+GQuark
+_cogl_error_quark (void)
+{
+  return g_quark_from_static_string ("cogl-error-quark");
+}
+
