@@ -14,9 +14,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,21 +26,26 @@
 #include <glib/gi18n-lib.h>
 
 #include <string.h>
-#include <sys/types.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 
-#include <sys/stat.h>
+#include <unistd.h>
+
 #include <sys/ioctl.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+
 #include <fcntl.h>
+
 #include <errno.h>
 
 #include "clutter-backend-x11.h"
+#include "clutter-device-manager-x11.h"
 #include "clutter-input-device-x11.h"
+#include "clutter-settings-x11.h"
 #include "clutter-stage-x11.h"
 #include "clutter-x11.h"
 
+#include "xsettings/xsettings-common.h"
+
 #include <X11/extensions/Xcomposite.h>
 
 #ifdef HAVE_XINPUT
@@ -48,6 +53,7 @@
 #endif
 
 #include "cogl/cogl.h"
+#include "cogl/cogl-internal.h"
 
 #include "../clutter-debug.h"
 #include "../clutter-device-manager.h"
@@ -95,127 +101,121 @@ static gboolean clutter_synchronise = FALSE;
 static int TrappedErrorCode = 0;
 static int (* old_error_handler) (Display *, XErrorEvent *);
 
-static void
-clutter_x11_register_input_devices (ClutterBackendX11 *backend)
+static ClutterX11FilterReturn
+xsettings_filter (XEvent       *xevent,
+                  ClutterEvent *event,
+                  gpointer      data)
 {
-  ClutterDeviceManager *manager;
-  ClutterInputDevice *device;
-#ifdef HAVE_XINPUT
-  XDeviceInfo *x_devices = NULL;
-  int res, opcode, event, error;
-  int i, n_devices;
-#endif /* HAVE_XINPUT */
+  ClutterBackendX11 *backend_x11 = data;
 
-  manager = clutter_device_manager_get_default ();
+  _clutter_xsettings_client_process_event (backend_x11->xsettings, xevent);
 
-  if (!clutter_enable_xinput)
+  /* we always want the rest of the stack to get XSettings events, even
+   * if Clutter already handled them
+   */
+
+  return CLUTTER_X11_FILTER_CONTINUE;
+}
+
+static Bool
+clutter_backend_x11_xsettings_watch (Window  window,
+                                     Bool    is_start,
+                                     long    mask,
+                                     void   *cb_data)
+{
+  ClutterBackendX11 *backend_x11 = cb_data;
+
+  if (is_start)
     {
-      CLUTTER_NOTE (BACKEND, "XInput support not enabled");
-      goto default_device;
+      backend_x11->xsettings_xwin = window;
+      if (!backend_x11->xsettings_xwin)
+        return False;
+
+      clutter_x11_add_filter (xsettings_filter, backend_x11);
+
+      CLUTTER_NOTE (BACKEND, "Added filter on XSettings manager window 0x%x",
+                    (unsigned int) backend_x11->xsettings_xwin);
+    }
+  else
+    {
+      CLUTTER_NOTE (BACKEND, "Removed filter on XSettings manager window 0x%x",
+                    (unsigned int) backend_x11->xsettings_xwin);
+
+      clutter_x11_remove_filter (xsettings_filter, backend_x11);
+
+      backend_x11->xsettings_xwin = None;
     }
 
-#ifdef HAVE_XINPUT
-  res = XQueryExtension (backend->xdpy, "XInputExtension",
-                         &opcode,
-                         &event,
-                         &error);
-  if (!res)
+  return True;
+}
+
+static void
+clutter_backend_x11_xsettings_notify (const char       *name,
+                                      XSettingsAction   action,
+                                      XSettingsSetting *setting,
+                                      void             *cb_data)
+{
+  ClutterSettings *settings = clutter_settings_get_default ();
+  gint i;
+
+  if (name == NULL || *name == '\0')
+    return;
+
+  g_object_freeze_notify (G_OBJECT (settings));
+
+  for (i = 0; i < _n_clutter_settings_map; i++)
     {
-      CLUTTER_NOTE (BACKEND, "No XInput extension available");
-      goto default_device;
-    }
-
-  backend->xi_event_base = event;
-
-  x_devices = XListInputDevices (backend->xdpy, &n_devices);
-  if (n_devices == 0)
-    {
-      CLUTTER_NOTE (BACKEND, "No XInput devices found");
-      goto default_device;
-    }
-
-  for (i = 0; i < n_devices; i++)
-    {
-      XDeviceInfo *info = x_devices + i;
-
-      CLUTTER_NOTE (BACKEND,
-                    "Considering device %li with type %d, %d of %d",
-                    info->id,
-                    info->use,
-                    i, n_devices);
-
-      /* we only want 'raw' devices, not virtual ones */
-      if (info->use == IsXExtensionPointer ||
-       /* info->use == IsXExtensionKeyboard || XInput1 is broken */
-          info->use == IsXExtensionDevice)
+      if (g_strcmp0 (name, CLUTTER_SETTING_X11_NAME (i)) == 0)
         {
-          ClutterInputDeviceType device_type;
-          gint n_events = 0;
+          GValue value = { 0, };
 
-          switch (info->use)
+          switch (setting->type)
             {
-            case IsXExtensionPointer:
-              device_type = CLUTTER_POINTER_DEVICE;
+            case XSETTINGS_TYPE_INT:
+              g_value_init (&value, G_TYPE_INT);
+              g_value_set_int (&value, setting->data.v_int);
               break;
 
-            /* XInput1 is broken for keyboards */
-            case IsXExtensionKeyboard:
-              device_type = CLUTTER_KEYBOARD_DEVICE;
+            case XSETTINGS_TYPE_STRING:
+              g_value_init (&value, G_TYPE_STRING);
+              g_value_set_string (&value, setting->data.v_string);
               break;
 
-            case IsXExtensionDevice:
-              device_type = CLUTTER_EXTENSION_DEVICE;
+            case XSETTINGS_TYPE_COLOR:
+              {
+                ClutterColor color;
+
+                color.red   = (guint8) ((float) setting->data.v_color.red
+                            / 65535.0 * 255);
+                color.green = (guint8) ((float) setting->data.v_color.green
+                            / 65535.0 * 255);
+                color.blue  = (guint8) ((float) setting->data.v_color.blue
+                            / 65535.0 * 255);
+                color.alpha = (guint8) ((float) setting->data.v_color.alpha
+                            / 65535.0 * 255);
+
+                g_value_init (&value, G_TYPE_BOXED);
+                clutter_value_set_color (&value, &color);
+              }
               break;
             }
 
-          device = g_object_new (CLUTTER_TYPE_INPUT_DEVICE_X11,
-                                 "id", info->id,
-                                 "device-type", device_type,
-                                 "name", info->name,
-                                 NULL);
-          n_events = _clutter_input_device_x11_construct (device, backend);
+          CLUTTER_NOTE (BACKEND,
+                        "Mapping XSETTING '%s' to 'ClutterSettings:%s'",
+                        CLUTTER_SETTING_X11_NAME (i),
+                        CLUTTER_SETTING_PROPERTY (i));
 
-          _clutter_device_manager_add_device (manager, device);
+          g_object_set_property (G_OBJECT (settings),
+                                 CLUTTER_SETTING_PROPERTY (i),
+                                 &value);
 
-          if (info->use == IsXExtensionPointer && n_events > 0)
-            backend->have_xinput = TRUE;
+          g_value_unset (&value);
+
+          break;
         }
     }
 
-  XFree (x_devices);
-#endif /* HAVE_XINPUT */
-
-default_device:
-  /* fallback code in case:
-   *
-   *  - we do not have XInput support compiled in
-   *  - we do not have XInput support enabled
-   *  - we do not have the XInput extension
-   *
-   * we register two default devices, one for the pointer
-   * and one for the keyboard. this block must also be
-   * executed for the XInput support because XI does not
-   * cover core devices
-   */
-  device = g_object_new (CLUTTER_TYPE_INPUT_DEVICE_X11,
-                         "id", 0,
-                         "name", "Core Pointer",
-                         "device-type", CLUTTER_POINTER_DEVICE,
-                         "is-core", TRUE,
-                         NULL);
-  CLUTTER_NOTE (BACKEND, "Added core pointer device");
-  _clutter_device_manager_add_device (manager, device);
-  backend->core_pointer = device;
-
-  device = g_object_new (CLUTTER_TYPE_INPUT_DEVICE_X11,
-                         "id", 1,
-                         "name", "Core Keyboard",
-                         "device-type", CLUTTER_KEYBOARD_DEVICE,
-                         "is-core", TRUE,
-                         NULL);
-  CLUTTER_NOTE (BACKEND, "Added core keyboard device");
-  _clutter_device_manager_add_device (manager, device);
-  backend->core_keyboard = device;
+  g_object_thaw_notify (G_OBJECT (settings));
 }
 
 gboolean
@@ -263,7 +263,7 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
 	  CLUTTER_NOTE (BACKEND, "XOpenDisplay on '%s'",
 			clutter_display_name);
 	  backend_x11->xdpy = XOpenDisplay (clutter_display_name);
-          if (backend_x11->xdpy == None)
+          if (backend_x11->xdpy == NULL)
             {
               g_set_error (error, CLUTTER_INIT_ERROR,
                            CLUTTER_INIT_ERROR_BACKEND,
@@ -290,6 +290,10 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
 
       CLUTTER_NOTE (BACKEND, "Getting the X screen");
 
+      /* Cogl needs to know the Xlib display connection for
+         CoglTexturePixmapX11 */
+      _cogl_xlib_set_display (backend_x11->xdpy);
+
       if (clutter_screen == -1)
         backend_x11->xscreen = DefaultScreenOfDisplay (backend_x11->xdpy);
       else
@@ -309,7 +313,19 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
       clutter_backend_set_resolution (backend, dpi);
 
       /* register input devices */
-      clutter_x11_register_input_devices (backend_x11);
+      backend_x11->device_manager =
+        g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_X11,
+                      "use-xinput-1", clutter_enable_xinput,
+                      "backend", backend_x11,
+                      NULL);
+
+      /* create XSETTINGS client */
+      backend_x11->xsettings =
+        _clutter_xsettings_client_new (backend_x11->xdpy,
+                                       backend_x11->xscreen_num,
+                                       clutter_backend_x11_xsettings_notify,
+                                       clutter_backend_x11_xsettings_watch,
+                                       backend_x11);
 
       if (clutter_synchronise)
         XSynchronize (backend_x11->xdpy, True);
@@ -398,6 +414,8 @@ clutter_backend_x11_finalize (GObject *gobject)
 
   g_free (backend_x11->display_name);
 
+  _clutter_xsettings_client_destroy (backend_x11->xsettings);
+
   XCloseDisplay (backend_x11->xdpy);
 
   if (backend_singleton)
@@ -464,6 +482,23 @@ clutter_backend_x11_handle_event (ClutterBackendX11 *backend_x11,
   return FALSE;
 }
 
+static ClutterDeviceManager *
+clutter_backend_x11_get_device_manager (ClutterBackend *backend)
+{
+  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+
+  if (G_UNLIKELY (backend_x11->device_manager == NULL))
+    {
+      backend_x11->device_manager =
+        g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_X11,
+                      "use-xinput-1", clutter_enable_xinput,
+                      "backend", backend_x11,
+                      NULL);
+    }
+
+  return backend_x11->device_manager;
+}
+
 static void
 clutter_backend_x11_class_init (ClutterBackendX11Class *klass)
 {
@@ -475,11 +510,12 @@ clutter_backend_x11_class_init (ClutterBackendX11Class *klass)
   gobject_class->dispose = clutter_backend_x11_dispose;
   gobject_class->finalize = clutter_backend_x11_finalize;
 
-  backend_class->pre_parse        = clutter_backend_x11_pre_parse;
-  backend_class->post_parse       = clutter_backend_x11_post_parse;
-  backend_class->init_events      = clutter_backend_x11_init_events;
-  backend_class->add_options      = clutter_backend_x11_add_options;
-  backend_class->get_features     = clutter_backend_x11_get_features;
+  backend_class->pre_parse = clutter_backend_x11_pre_parse;
+  backend_class->post_parse = clutter_backend_x11_post_parse;
+  backend_class->init_events = clutter_backend_x11_init_events;
+  backend_class->add_options = clutter_backend_x11_add_options;
+  backend_class->get_features = clutter_backend_x11_get_features;
+  backend_class->get_device_manager = clutter_backend_x11_get_device_manager;
 
   backendx11_class->handle_event = clutter_backend_x11_handle_event;
 }
@@ -487,13 +523,6 @@ clutter_backend_x11_class_init (ClutterBackendX11Class *klass)
 static void
 clutter_backend_x11_init (ClutterBackendX11 *backend_x11)
 {
-  ClutterBackend *backend = CLUTTER_BACKEND (backend_x11);
-
-  /* FIXME: get from xsettings */
-  clutter_backend_set_double_click_time (backend, 250);
-  clutter_backend_set_double_click_distance (backend, 5);
-  clutter_backend_set_resolution (backend, 96.0);
-
   backend_x11->last_event_time = CurrentTime;
 }
 
@@ -589,11 +618,13 @@ clutter_x11_set_display (Display *xdpy)
  * clutter_x11_enable_xinput:
  *
  * Enables the use of the XInput extension if present on connected
- * XServer and support built into Clutter.  XInput allows for multiple
- * pointing devices to be used. This must be called before
- * clutter_init().
+ * XServer and support built into Clutter. XInput allows for multiple
+ * pointing devices to be used.
  *
- * You should use #clutter_x11_has_xinput to see if support was enabled.
+ * This function must be called before clutter_init().
+ *
+ * Since XInput might not be supported by the X server, you might
+ * want to use clutter_x11_has_xinput() to see if support was enabled.
  *
  * Since: 0.8
  */
@@ -848,6 +879,14 @@ clutter_x11_has_xinput (void)
 #endif
 }
 
+/**
+ * clutter_x11_has_composite_extension:
+ *
+ * Retrieves whether Clutter is running on an X11 server with the
+ * XComposite extension
+ *
+ * Return value: %TRUE if the XComposite extension is available
+ */
 gboolean
 clutter_x11_has_composite_extension (void)
 {
